__all__ = ["AsyncClient", "AsyncDriver", "Client", "Driver", "Websocket", "AsyncTypedDriver", "TypedDriver"]
from .driver import Driver, AsyncDriver, TypedDriver, AsyncTypedDriver
from .client import Client, AsyncClient
from .websocket import Websocket



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Internal"]


class Internal(Base):

    def create_playbook_run_from_dialog(
        self,
        type: str | None = None,
        url: str | None = None,
        callback_id: str | None = None,
        state: str | None = None,
        user_id: str | None = None,
        channel_id: str | None = None,
        team_id: str | None = None,
        submission: dict[str, Any] | None = None,
        cancelled: bool | None = None,
    ):
        """Create a new playbook run from dialog

        type:
        url:
        callback_id: Callback ID provided by the integration.
        state: Stringified JSON with the post_id and the client_id.
        user_id: ID of the user who submitted the dialog.
        channel_id: ID of the channel the user was in when submitting the dialog.
        team_id: ID of the team the user was on when submitting the dialog.
        submission: Map of the dialog fields to their values
        cancelled: If the dialog was cancelled.

        `Read in Mattermost API docs (internal - createPlaybookRunFromDialog) <https://developers.mattermost.com/api-documentation/#/operations/createPlaybookRunFromDialog>`_

        """
        __options = {
            "type": type,
            "url": url,
            "callback_id": callback_id,
            "state": state,
            "user_id": user_id,
            "channel_id": channel_id,
            "team_id": team_id,
            "submission": submission,
            "cancelled": cancelled,
        }
        return self.client.post("""/plugins/playbooks/api/v0/runs/dialog""", options=__options)

    def get_checklist_autocomplete(self, channel_ID: str):
        """Get autocomplete data for /playbook check

        channel_ID: ID of the channel the user is in.

        `Read in Mattermost API docs (internal - getChecklistAutocomplete) <https://developers.mattermost.com/api-documentation/#/operations/getChecklistAutocomplete>`_

        """
        __params = {"channel_ID": channel_ID}
        return self.client.get("""/plugins/playbooks/api/v0/runs/checklist-autocomplete""", params=__params)

    def end_playbook_run_dialog(self, id: str):
        """End a playbook run from dialog

        id: ID of the playbook run to end.

        `Read in Mattermost API docs (internal - endPlaybookRunDialog) <https://developers.mattermost.com/api-documentation/#/operations/endPlaybookRunDialog>`_

        """
        return self.client.post(f"/plugins/playbooks/api/v0/runs/{id}/end")

    def next_stage_dialog(self, id: str, state: str | None = None):
        """Go to next stage from dialog

        id: The PlaybookRun ID
        state: String representation of the zero-based index of the stage to go to.

        `Read in Mattermost API docs (internal - nextStageDialog) <https://developers.mattermost.com/api-documentation/#/operations/nextStageDialog>`_

        """
        __options = {"state": state}
        return self.client.post(f"/plugins/playbooks/api/v0/runs/{id}/next-stage-dialog", options=__options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["ScheduledPost"]


class ScheduledPost(Base):

    def create_scheduled_post(
        self,
        scheduled_at: int,
        channel_id: str,
        message: str,
        root_id: str | None = None,
        file_ids: list[Any] | None = None,
        props: dict[str, Any] | None = None,
    ):
        """Creates a scheduled post

        scheduled_at: UNIX timestamp in milliseconds of the time when the scheduled post should be sent
        channel_id: The channel ID to post in
        message: The message contents, can be formatted with Markdown
        root_id: The post ID to comment on
        file_ids: A list of file IDs to associate with the post. Note that posts are limited to 5 files maximum. Please use additional posts for more files.
        props: A general JSON property bag to attach to the post

        `Read in Mattermost API docs (scheduled_post - CreateScheduledPost) <https://developers.mattermost.com/api-documentation/#/operations/CreateScheduledPost>`_

        """
        __options = {
            "scheduled_at": scheduled_at,
            "channel_id": channel_id,
            "message": message,
            "root_id": root_id,
            "file_ids": file_ids,
            "props": props,
        }
        return self.client.post("""/api/v4/posts/schedule""", options=__options)

    def get_user_scheduled_posts(self, includeDirectChannels: bool | None = False):
        """Gets all scheduled posts for a user for the specified team..

        includeDirectChannels: Whether to include scheduled posts from DMs an GMs or not. Default is false

        `Read in Mattermost API docs (scheduled_post - GetUserScheduledPosts) <https://developers.mattermost.com/api-documentation/#/operations/GetUserScheduledPosts>`_

        """
        __params = {"includeDirectChannels": includeDirectChannels}
        return self.client.get(f"/api/v4/posts/scheduled/team/{team_id}", params=__params)

    def update_scheduled_post(
        self, scheduled_post_id: str, id: str, channel_id: str, user_id: str, scheduled_at: int, message: str
    ):
        """Update a scheduled post

        scheduled_post_id: ID of the scheduled post to update
        id: ID of the scheduled post to update
        channel_id: The channel ID to post in
        user_id: The current user ID
        scheduled_at: UNIX timestamp in milliseconds of the time when the scheduled post should be sent
        message: The message contents, can be formatted with Markdown

        `Read in Mattermost API docs (scheduled_post - UpdateScheduledPost) <https://developers.mattermost.com/api-documentation/#/operations/UpdateScheduledPost>`_

        """
        __options = {
            "id": id,
            "channel_id": channel_id,
            "user_id": user_id,
            "scheduled_at": scheduled_at,
            "message": message,
        }
        return self.client.put(f"/api/v4/posts/schedule/{scheduled_post_id}", options=__options)

    def delete_scheduled_post(self, scheduled_post_id: str):
        """Delete a scheduled post

        scheduled_post_id: ID of the scheduled post to delete

        `Read in Mattermost API docs (scheduled_post - DeleteScheduledPost) <https://developers.mattermost.com/api-documentation/#/operations/DeleteScheduledPost>`_

        """
        return self.client.delete(f"/api/v4/posts/schedule/{scheduled_post_id}")



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["RemoteClusters"]


class RemoteClusters(Base):

    def get_remote_clusters(
        self,
        page: int | None = None,
        per_page: int | None = None,
        exclude_offline: bool | None = None,
        in_channel: str | None = None,
        not_in_channel: str | None = None,
        only_confirmed: bool | None = None,
        only_plugins: bool | None = None,
        exclude_plugins: bool | None = None,
        include_deleted: bool | None = None,
    ):
        """Get a list of remote clusters.

        page: The page to select
        per_page: The number of remote clusters per page
        exclude_offline: Exclude offline remote clusters
        in_channel: Select remote clusters in channel
        not_in_channel: Select remote clusters not in this channel
        only_confirmed: Select only remote clusters already confirmed
        only_plugins: Select only remote clusters that belong to a plugin
        exclude_plugins: Select only remote clusters that don't belong to a plugin
        include_deleted: Include those remote clusters that have been deleted

        `Read in Mattermost API docs (remote_clusters - GetRemoteClusters) <https://developers.mattermost.com/api-documentation/#/operations/GetRemoteClusters>`_

        """
        __params = {
            "page": page,
            "per_page": per_page,
            "exclude_offline": exclude_offline,
            "in_channel": in_channel,
            "not_in_channel": not_in_channel,
            "only_confirmed": only_confirmed,
            "only_plugins": only_plugins,
            "exclude_plugins": exclude_plugins,
            "include_deleted": include_deleted,
        }
        return self.client.get("""/api/v4/remotecluster""", params=__params)

    def create_remote_cluster(
        self, name: str, default_team_id: str, display_name: str | None = None, password: str | None = None
    ):
        """Create a new remote cluster.

        name:
        display_name:
        default_team_id:
        password: The password to use in the invite code. If empty,
        the server will generate one and it will be part
        of the response


        `Read in Mattermost API docs (remote_clusters - CreateRemoteCluster) <https://developers.mattermost.com/api-documentation/#/operations/CreateRemoteCluster>`_

        """
        __options = {
            "name": name,
            "display_name": display_name,
            "default_team_id": default_team_id,
            "password": password,
        }
        return self.client.post("""/api/v4/remotecluster""", options=__options)

    def get_remote_cluster(self, remote_id: str):
        """Get a remote cluster.

        remote_id: Remote Cluster GUID

        `Read in Mattermost API docs (remote_clusters - GetRemoteCluster) <https://developers.mattermost.com/api-documentation/#/operations/GetRemoteCluster>`_

        """
        return self.client.get(f"/api/v4/remotecluster/{remote_id}")

    def patch_remote_cluster(self, remote_id: str, display_name: str | None = None, default_team_id: str | None = None):
        """Patch a remote cluster.

        remote_id: Remote Cluster GUID
        display_name:
        default_team_id: The team where channels from invites are created

        `Read in Mattermost API docs (remote_clusters - PatchRemoteCluster) <https://developers.mattermost.com/api-documentation/#/operations/PatchRemoteCluster>`_

        """
        __options = {"display_name": display_name, "default_team_id": default_team_id}
        return self.client.patch(f"/api/v4/remotecluster/{remote_id}", options=__options)

    def delete_remote_cluster(self, remote_id: str):
        """Delete a remote cluster.

        remote_id: Remote Cluster GUID

        `Read in Mattermost API docs (remote_clusters - DeleteRemoteCluster) <https://developers.mattermost.com/api-documentation/#/operations/DeleteRemoteCluster>`_

        """
        return self.client.delete(f"/api/v4/remotecluster/{remote_id}")

    def generate_remote_cluster_invite(self, password: str):
        """Generate invite code.

        password: The password to encrypt the invite code with.

        `Read in Mattermost API docs (remote_clusters - GenerateRemoteClusterInvite) <https://developers.mattermost.com/api-documentation/#/operations/GenerateRemoteClusterInvite>`_

        """
        __options = {"password": password}
        return self.client.post(f"/api/v4/remotecluster/{remote_id}/generate_invite", options=__options)

    def accept_remote_cluster_invite(
        self, invite: str, name: str, default_team_id: str, password: str, display_name: str | None = None
    ):
        """Accept a remote cluster invite code.

        invite:
        name:
        display_name:
        default_team_id:
        password: The password to decrypt the invite code.

        `Read in Mattermost API docs (remote_clusters - AcceptRemoteClusterInvite) <https://developers.mattermost.com/api-documentation/#/operations/AcceptRemoteClusterInvite>`_

        """
        __options = {
            "invite": invite,
            "name": name,
            "display_name": display_name,
            "default_team_id": default_team_id,
            "password": password,
        }
        return self.client.post("""/api/v4/remotecluster/accept_invite""", options=__options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Filtering"]


class Filtering(Base):

    def get_ip_filters(self):
        """Get all IP filters
        `Read in Mattermost API docs (filtering - GetIPFilters) <https://developers.mattermost.com/api-documentation/#/operations/GetIPFilters>`_

        """
        return self.client.get("""/api/v4/ip_filtering""")

    def apply_ip_filters(self, options: list[Any]):
        """Get all IP filters
        `Read in Mattermost API docs (filtering - ApplyIPFilters) <https://developers.mattermost.com/api-documentation/#/operations/ApplyIPFilters>`_

        """
        return self.client.post("""/api/v4/ip_filtering""", options=options)

    def my_ip(self):
        """Get all IP filters
        `Read in Mattermost API docs (filtering - MyIP) <https://developers.mattermost.com/api-documentation/#/operations/MyIP>`_

        """
        return self.client.get("""/api/v4/ip_filtering/my_ip""")



from ._base import Base

__all__ = ["Emoji"]


class Emoji(Base):

    def create_emoji(self, files, data=None):
        """Create a custom emoji

        image: A file to be uploaded
        emoji: A JSON object containing a ``name`` field with the name of the emoji and a ``creator_id`` field with the id of the authenticated user.

        `Read in Mattermost API docs (emoji - CreateEmoji) <https://developers.mattermost.com/api-documentation/#/operations/CreateEmoji>`_

        """
        return self.client.post("""/api/v4/emoji""", files=files, data=data)

    def get_emoji_list(self, params=None):
        """Get a list of custom emoji

        page: The page to select.
        per_page: The number of emojis per page.
        sort: Either blank for no sorting or "name" to sort by emoji names. Minimum server version for sorting is 4.7.

        `Read in Mattermost API docs (emoji - GetEmojiList) <https://developers.mattermost.com/api-documentation/#/operations/GetEmojiList>`_

        """
        return self.client.get("""/api/v4/emoji""", params=params)

    def get_emoji(self, emoji_id):
        """Get a custom emoji

        emoji_id: Emoji GUID

        `Read in Mattermost API docs (emoji - GetEmoji) <https://developers.mattermost.com/api-documentation/#/operations/GetEmoji>`_

        """
        return self.client.get(f"/api/v4/emoji/{emoji_id}")

    def delete_emoji(self, emoji_id):
        """Delete a custom emoji

        emoji_id: Emoji GUID

        `Read in Mattermost API docs (emoji - DeleteEmoji) <https://developers.mattermost.com/api-documentation/#/operations/DeleteEmoji>`_

        """
        return self.client.delete(f"/api/v4/emoji/{emoji_id}")

    def get_emoji_by_name(self, emoji_name):
        """Get a custom emoji by name

        emoji_name: Emoji name

        `Read in Mattermost API docs (emoji - GetEmojiByName) <https://developers.mattermost.com/api-documentation/#/operations/GetEmojiByName>`_

        """
        return self.client.get(f"/api/v4/emoji/name/{emoji_name}")

    def get_emoji_image(self, emoji_id):
        """Get custom emoji image

        emoji_id: Emoji GUID

        `Read in Mattermost API docs (emoji - GetEmojiImage) <https://developers.mattermost.com/api-documentation/#/operations/GetEmojiImage>`_

        """
        return self.client.get(f"/api/v4/emoji/{emoji_id}/image")

    def search_emoji(self, options):
        """Search custom emoji

        term: The term to match against the emoji name.
        prefix_only: Set to only search for names starting with the search term.

        `Read in Mattermost API docs (emoji - SearchEmoji) <https://developers.mattermost.com/api-documentation/#/operations/SearchEmoji>`_

        """
        return self.client.post("""/api/v4/emoji/search""", options=options)

    def autocomplete_emoji(self, params=None):
        """Autocomplete custom emoji

        name: The emoji name to search.

        `Read in Mattermost API docs (emoji - AutocompleteEmoji) <https://developers.mattermost.com/api-documentation/#/operations/AutocompleteEmoji>`_

        """
        return self.client.get("""/api/v4/emoji/autocomplete""", params=params)

    def get_emojis_by_names(self, options):
        """Get custom emojis by name
        `Read in Mattermost API docs (emoji - GetEmojisByNames) <https://developers.mattermost.com/api-documentation/#/operations/GetEmojisByNames>`_

        """
        return self.client.post("""/api/v4/emoji/names""", options=options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Permissions"]


class Permissions(Base):

    def get_ancillary_permissions_post(self, options: list[str]):
        """Return all system console subsection ancillary permissions
        `Read in Mattermost API docs (permissions - GetAncillaryPermissionsPost) <https://developers.mattermost.com/api-documentation/#/operations/GetAncillaryPermissionsPost>`_

        """
        return self.client.post("""/api/v4/permissions/ancillary""", options=options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Bookmarks"]


class Bookmarks(Base):

    def list_channel_bookmarks_for_channel(self, channel_id: str, bookmarks_since: float | None = None):
        """Get channel bookmarks for Channel

        channel_id: Channel GUID
        bookmarks_since: Timestamp to filter the bookmarks with. If set, the
        endpoint returns bookmarks that have been added, updated
        or deleted since its value


        `Read in Mattermost API docs (bookmarks - ListChannelBookmarksForChannel) <https://developers.mattermost.com/api-documentation/#/operations/ListChannelBookmarksForChannel>`_

        """
        __params = {"bookmarks_since": bookmarks_since}
        return self.client.get(f"/api/v4/channels/{channel_id}/bookmarks", params=__params)

    def create_channel_bookmark(
        self,
        channel_id: str,
        display_name: str,
        type: str,
        file_id: str | None = None,
        link_url: str | None = None,
        image_url: str | None = None,
        emoji: str | None = None,
    ):
        """Create channel bookmark

        channel_id: Channel GUID
        file_id: The ID of the file associated with the channel bookmark. Required for bookmarks of type 'file'
        display_name: The name of the channel bookmark
        link_url: The URL associated with the channel bookmark. Required for bookmarks of type 'link'
        image_url: The URL of the image associated with the channel bookmark. Optional, only applies for bookmarks of type 'link'
        emoji: The emoji of the channel bookmark
        type: * ``link`` for channel bookmarks that reference a link. ``link_url`` is requied
        * ``file`` for channel bookmarks that reference a file. ``file_id`` is required


        `Read in Mattermost API docs (bookmarks - CreateChannelBookmark) <https://developers.mattermost.com/api-documentation/#/operations/CreateChannelBookmark>`_

        """
        __options = {
            "file_id": file_id,
            "display_name": display_name,
            "link_url": link_url,
            "image_url": image_url,
            "emoji": emoji,
            "type": type,
        }
        return self.client.post(f"/api/v4/channels/{channel_id}/bookmarks", options=__options)

    def update_channel_bookmark(
        self,
        channel_id: str,
        bookmark_id: str,
        file_id: str | None = None,
        display_name: str | None = None,
        sort_order: int | None = None,
        link_url: str | None = None,
        image_url: str | None = None,
        emoji: str | None = None,
        type: str | None = None,
    ):
        """Update channel bookmark

        channel_id: Channel GUID
        bookmark_id: Bookmark GUID
        file_id: The ID of the file associated with the channel bookmark. Required for bookmarks of type 'file'
        display_name: The name of the channel bookmark
        sort_order: The order of the channel bookmark
        link_url: The URL associated with the channel bookmark. Required for type bookmarks of type 'link'
        image_url: The URL of the image associated with the channel bookmark
        emoji: The emoji of the channel bookmark
        type: * ``link`` for channel bookmarks that reference a link. ``link_url`` is requied
        * ``file`` for channel bookmarks that reference a file. ``file_id`` is required


        `Read in Mattermost API docs (bookmarks - UpdateChannelBookmark) <https://developers.mattermost.com/api-documentation/#/operations/UpdateChannelBookmark>`_

        """
        __options = {
            "file_id": file_id,
            "display_name": display_name,
            "sort_order": sort_order,
            "link_url": link_url,
            "image_url": image_url,
            "emoji": emoji,
            "type": type,
        }
        return self.client.patch(f"/api/v4/channels/{channel_id}/bookmarks/{bookmark_id}", options=__options)

    def delete_channel_bookmark(self, channel_id: str, bookmark_id: str):
        """Delete channel bookmark

        channel_id: Channel GUID
        bookmark_id: Bookmark GUID

        `Read in Mattermost API docs (bookmarks - DeleteChannelBookmark) <https://developers.mattermost.com/api-documentation/#/operations/DeleteChannelBookmark>`_

        """
        return self.client.delete(f"/api/v4/channels/{channel_id}/bookmarks/{bookmark_id}")

    def update_channel_bookmark_sort_order(self, channel_id: str, bookmark_id: str, options: float | None = None):
        """Update channel bookmark's order

        channel_id: Channel GUID
        bookmark_id: Bookmark GUID

        `Read in Mattermost API docs (bookmarks - UpdateChannelBookmarkSortOrder) <https://developers.mattermost.com/api-documentation/#/operations/UpdateChannelBookmarkSortOrder>`_

        """
        return self.client.post(f"/api/v4/channels/{channel_id}/bookmarks/{bookmark_id}/sort_order", options=options)



from ._base import Base

__all__ = ["PlaybookRuns"]


class PlaybookRuns(Base):

    def list_playbook_runs(self, params=None):
        """List all playbook runs

        team_id: ID of the team to filter by.
        page: Zero-based index of the page to request.
        per_page: Number of playbook runs to return per page.
        sort: Field to sort the returned playbook runs by.
        direction: Direction (ascending or descending) followed by the sorting of the playbook runs.
        statuses: The returned list will contain only the playbook runs with the specified statuses.
        owner_user_id: The returned list will contain only the playbook runs commanded by this user. Specify "me" for current user.
        participant_id: The returned list will contain only the playbook runs for which the given user is a participant. Specify "me" for current user.
        search_term: The returned list will contain only the playbook runs whose name contains the search term.
        channel_id: The returned list will contain only the playbook runs associated with this channel ID.
        omit_ended: When set to true, only active runs (with EndAt = 0) are returned. When false or omitted, both active and ended runs are returned.
        since: Return only PlaybookRuns created/modified since the given timestamp (in milliseconds).

        `Read in Mattermost API docs (playbook_runs - listPlaybookRuns) <https://developers.mattermost.com/api-documentation/#/operations/listPlaybookRuns>`_

        """
        return self.client.get("""/plugins/playbooks/api/v0/runs""", params=params)

    def create_playbook_run_from_post(self, options=None):
        """Create a new playbook run

        name: The name of the playbook run.
        summary: The summary of the playbook run.
        owner_user_id: The identifier of the user who is commanding the playbook run.
        team_id: The identifier of the team where the playbook run's channel is in.
        post_id: If the playbook run was created from a post, this field contains the identifier of such post. If not, this field is empty.
        playbook_id: The identifier of the playbook with from which this playbook run was created.

        `Read in Mattermost API docs (playbook_runs - createPlaybookRunFromPost) <https://developers.mattermost.com/api-documentation/#/operations/createPlaybookRunFromPost>`_

        """
        return self.client.post("""/plugins/playbooks/api/v0/runs""", options=options)

    def get_owners(self, params=None):
        """Get all owners

        team_id: ID of the team to filter by.

        `Read in Mattermost API docs (playbook_runs - getOwners) <https://developers.mattermost.com/api-documentation/#/operations/getOwners>`_

        """
        return self.client.get("""/plugins/playbooks/api/v0/runs/owners""", params=params)

    def get_channels(self, params=None):
        """Get playbook run channels

        team_id: ID of the team to filter by.
        sort: Field to sort the returned channels by, according to their playbook run.
        direction: Direction (ascending or descending) followed by the sorting of the playbook runs associated to the channels.
        status: The returned list will contain only the channels whose playbook run has this status.
        owner_user_id: The returned list will contain only the channels whose playbook run is commanded by this user.
        search_term: The returned list will contain only the channels associated to a playbook run whose name contains the search term.
        participant_id: The returned list will contain only the channels associated to a playbook run for which the given user is a participant.

        `Read in Mattermost API docs (playbook_runs - getChannels) <https://developers.mattermost.com/api-documentation/#/operations/getChannels>`_

        """
        return self.client.get("""/plugins/playbooks/api/v0/runs/channels""", params=params)

    def get_playbook_run_by_channel_id(self, channel_id):
        """Find playbook run by channel ID

        channel_id: ID of the channel associated to the playbook run to retrieve.

        `Read in Mattermost API docs (playbook_runs - getPlaybookRunByChannelId) <https://developers.mattermost.com/api-documentation/#/operations/getPlaybookRunByChannelId>`_

        """
        return self.client.get(f"/plugins/playbooks/api/v0/runs/channel/{channel_id}")

    def get_playbook_run(self, id):
        """Get a playbook run

        id: ID of the playbook run to retrieve.

        `Read in Mattermost API docs (playbook_runs - getPlaybookRun) <https://developers.mattermost.com/api-documentation/#/operations/getPlaybookRun>`_

        """
        return self.client.get(f"/plugins/playbooks/api/v0/runs/{id}")

    def update_playbook_run(self, id, options=None):
        """Update a playbook run

        id: ID of the playbook run to retrieve.
        name: The new name of the playbook run. Must not be empty.
        summary: The new summary of the playbook run. Can be empty to clear the summary.

        `Read in Mattermost API docs (playbook_runs - updatePlaybookRun) <https://developers.mattermost.com/api-documentation/#/operations/updatePlaybookRun>`_

        """
        return self.client.patch(f"/plugins/playbooks/api/v0/runs/{id}", options=options)

    def get_playbook_run_metadata(self, id):
        """Get playbook run metadata

        id: ID of the playbook run whose metadata will be retrieved.

        `Read in Mattermost API docs (playbook_runs - getPlaybookRunMetadata) <https://developers.mattermost.com/api-documentation/#/operations/getPlaybookRunMetadata>`_

        """
        return self.client.get(f"/plugins/playbooks/api/v0/runs/{id}/metadata")

    def end_playbook_run(self, id):
        """End a playbook run

        id: ID of the playbook run to end.

        `Read in Mattermost API docs (playbook_runs - endPlaybookRun) <https://developers.mattermost.com/api-documentation/#/operations/endPlaybookRun>`_

        """
        return self.client.put(f"/plugins/playbooks/api/v0/runs/{id}/end")

    def restart_playbook_run(self, id):
        """Restart a playbook run

        id: ID of the playbook run to restart.

        `Read in Mattermost API docs (playbook_runs - restartPlaybookRun) <https://developers.mattermost.com/api-documentation/#/operations/restartPlaybookRun>`_

        """
        return self.client.put(f"/plugins/playbooks/api/v0/runs/{id}/restart")

    def status(self, id, options=None):
        """Update a playbook run's status

        id: ID of the playbook run to update.
        message: The status update message.
        reminder: The number of seconds until the system will send a reminder to the owner to update the status. No reminder will be scheduled if reminder is 0 or omitted.

        `Read in Mattermost API docs (playbook_runs - status) <https://developers.mattermost.com/api-documentation/#/operations/status>`_

        """
        return self.client.post(f"/plugins/playbooks/api/v0/runs/{id}/status", options=options)

    def finish(self, id):
        """Finish a playbook

        id: ID of the playbook run to finish.

        `Read in Mattermost API docs (playbook_runs - finish) <https://developers.mattermost.com/api-documentation/#/operations/finish>`_

        """
        return self.client.put(f"/plugins/playbooks/api/v0/runs/{id}/finish")

    def change_owner(self, id, options=None):
        """Update playbook run owner

        id: ID of the playbook run whose owner will be changed.
        owner_id: The user ID of the new owner.

        `Read in Mattermost API docs (playbook_runs - changeOwner) <https://developers.mattermost.com/api-documentation/#/operations/changeOwner>`_

        """
        return self.client.post(f"/plugins/playbooks/api/v0/runs/{id}/owner", options=options)

    def add_checklist_item(self, id, checklist, options=None):
        """Add an item to a playbook run's checklist

        id: ID of the playbook run whose checklist will be modified.
        checklist: Zero-based index of the checklist to modify.
        title: The title of the checklist item.
        state: The state of the checklist item. An empty string means that the item is not done.
        state_modified: The timestamp for the latest modification of the item's state, formatted as the number of milliseconds since the Unix epoch. It equals 0 if the item was never modified.
        assignee_id: The identifier of the user that has been assigned to complete this item. If the item has no assignee, this is an empty string.
        assignee_modified: The timestamp for the latest modification of the item's assignee, formatted as the number of milliseconds since the Unix epoch. It equals 0 if the item never got an assignee.
        command: The slash command associated with this item. If the item has no slash command associated, this is an empty string
        command_last_run: The timestamp for the latest execution of the item's command, formatted as the number of milliseconds since the Unix epoch. It equals 0 if the command was never executed.
        description: A detailed description of the checklist item, formatted with Markdown.

        `Read in Mattermost API docs (playbook_runs - addChecklistItem) <https://developers.mattermost.com/api-documentation/#/operations/addChecklistItem>`_

        """
        return self.client.post(f"/plugins/playbooks/api/v0/runs/{id}/checklists/{checklist}/add", options=options)

    def reoder_checklist_item(self, id, checklist, options=None):
        """Reorder an item in a playbook run's checklist

        id: ID of the playbook run whose checklist will be modified.
        checklist: Zero-based index of the checklist to modify.
        item_num: Zero-based index of the item to reorder.
        new_location: Zero-based index of the new place to move the item to.

        `Read in Mattermost API docs (playbook_runs - reoderChecklistItem) <https://developers.mattermost.com/api-documentation/#/operations/reoderChecklistItem>`_

        """
        return self.client.put(f"/plugins/playbooks/api/v0/runs/{id}/checklists/{checklist}/reorder", options=options)

    def item_rename(self, id, checklist, item, options=None):
        """Update an item of a playbook run's checklist

        id: ID of the playbook run whose checklist will be modified.
        checklist: Zero-based index of the checklist to modify.
        item: Zero-based index of the item to modify.
        title: The new title of the item.
        command: The new slash command of the item.
        description: The new description of the item, formatted with Markdown.

        `Read in Mattermost API docs (playbook_runs - itemRename) <https://developers.mattermost.com/api-documentation/#/operations/itemRename>`_

        """
        return self.client.put(
            f"/plugins/playbooks/api/v0/runs/{id}/checklists/{checklist}/item/{item}", options=options
        )

    def item_delete(self, id, checklist, item):
        """Delete an item of a playbook run's checklist

        id: ID of the playbook run whose checklist will be modified.
        checklist: Zero-based index of the checklist to modify.
        item: Zero-based index of the item to modify.

        `Read in Mattermost API docs (playbook_runs - itemDelete) <https://developers.mattermost.com/api-documentation/#/operations/itemDelete>`_

        """
        return self.client.delete(f"/plugins/playbooks/api/v0/runs/{id}/checklists/{checklist}/item/{item}")

    def item_set_state(self, id, checklist, item, options=None):
        """Update the state of an item

        id: ID of the playbook run whose checklist will be modified.
        checklist: Zero-based index of the checklist to modify.
        item: Zero-based index of the item to modify.
        new_state: The new state of the item.

        `Read in Mattermost API docs (playbook_runs - itemSetState) <https://developers.mattermost.com/api-documentation/#/operations/itemSetState>`_

        """
        return self.client.put(
            f"/plugins/playbooks/api/v0/runs/{id}/checklists/{checklist}/item/{item}/state", options=options
        )

    def item_set_assignee(self, id, checklist, item, options=None):
        """Update the assignee of an item

        id: ID of the playbook run whose item will get a new assignee.
        checklist: Zero-based index of the checklist whose item will get a new assignee.
        item: Zero-based index of the item that will get a new assignee.
        assignee_id: The user ID of the new assignee of the item.

        `Read in Mattermost API docs (playbook_runs - itemSetAssignee) <https://developers.mattermost.com/api-documentation/#/operations/itemSetAssignee>`_

        """
        return self.client.put(
            f"/plugins/playbooks/api/v0/runs/{id}/checklists/{checklist}/item/{item}/assignee", options=options
        )

    def item_run(self, id, checklist, item):
        """Run an item's slash command

        id: ID of the playbook run whose item will be executed.
        checklist: Zero-based index of the checklist whose item will be executed.
        item: Zero-based index of the item whose slash command will be executed.

        `Read in Mattermost API docs (playbook_runs - itemRun) <https://developers.mattermost.com/api-documentation/#/operations/itemRun>`_

        """
        return self.client.put(f"/plugins/playbooks/api/v0/runs/{id}/checklists/{checklist}/item/{item}/run")

    def get_run_property_fields(self, id, params=None):
        """Get property fields for a playbook run

        id: ID of the playbook run to retrieve property fields from.
        updated_since: Filter results to only include property fields updated after this timestamp (Unix time in milliseconds).

        `Read in Mattermost API docs (playbook_runs - getRunPropertyFields) <https://developers.mattermost.com/api-documentation/#/operations/getRunPropertyFields>`_

        """
        return self.client.get(f"/plugins/playbooks/api/v0/runs/{id}/property_fields", params=params)

    def get_run_property_values(self, id, params=None):
        """Get property values for a playbook run

        id: ID of the playbook run to retrieve property values from.
        updated_since: Filter results to only include property values updated after this timestamp (Unix time in milliseconds).

        `Read in Mattermost API docs (playbook_runs - getRunPropertyValues) <https://developers.mattermost.com/api-documentation/#/operations/getRunPropertyValues>`_

        """
        return self.client.get(f"/plugins/playbooks/api/v0/runs/{id}/property_values", params=params)

    def set_run_property_value(self, id, field_id, options=None):
        """Set a property value for a playbook run

        id: ID of the playbook run to set property value for.
        field_id: ID of the property field to set value for.

        `Read in Mattermost API docs (playbook_runs - setRunPropertyValue) <https://developers.mattermost.com/api-documentation/#/operations/setRunPropertyValue>`_

        """
        return self.client.put(f"/plugins/playbooks/api/v0/runs/{id}/property_fields/{field_id}/value", options=options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Webhooks"]


class Webhooks(Base):

    def create_incoming_webhook(
        self,
        channel_id: str,
        user_id: str | None = None,
        display_name: str | None = None,
        description: str | None = None,
        username: str | None = None,
        icon_url: str | None = None,
        channel_locked: bool | None = None,
    ):
        """Create an incoming webhook

        channel_id: The ID of a public channel or private group that receives the webhook payloads.
        user_id: The ID of the owner of the webhook if different than the requester. Required for [local mode](https://docs.mattermost.com/administration/mmctl-cli-tool.html#local-mode).
        display_name: The display name for this incoming webhook
        description: The description for this incoming webhook
        username: The username this incoming webhook will post as.
        icon_url: The profile picture this incoming webhook will use when posting.
        channel_locked: Whether the webhook is locked to the channel.

        `Read in Mattermost API docs (webhooks - CreateIncomingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/CreateIncomingWebhook>`_

        """
        __options = {
            "channel_id": channel_id,
            "user_id": user_id,
            "display_name": display_name,
            "description": description,
            "username": username,
            "icon_url": icon_url,
            "channel_locked": channel_locked,
        }
        return self.client.post("""/api/v4/hooks/incoming""", options=__options)

    def get_incoming_webhooks(
        self,
        page: int | None = 0,
        per_page: int | None = 60,
        team_id: str | None = None,
        include_total_count: bool | None = False,
    ):
        """List incoming webhooks

        page: The page to select.
        per_page: The number of hooks per page.
        team_id: The ID of the team to get hooks for.
        include_total_count: Appends a total count of returned hooks inside the response object - ex: ``{ "incoming_webhooks": [], "total_count": 0 }``.

        `Read in Mattermost API docs (webhooks - GetIncomingWebhooks) <https://developers.mattermost.com/api-documentation/#/operations/GetIncomingWebhooks>`_

        """
        __params = {"page": page, "per_page": per_page, "team_id": team_id, "include_total_count": include_total_count}
        return self.client.get("""/api/v4/hooks/incoming""", params=__params)

    def get_incoming_webhook(self, hook_id: str):
        """Get an incoming webhook

        hook_id: Incoming Webhook GUID

        `Read in Mattermost API docs (webhooks - GetIncomingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/GetIncomingWebhook>`_

        """
        return self.client.get(f"/api/v4/hooks/incoming/{hook_id}")

    def delete_incoming_webhook(self, hook_id: str):
        """Delete an incoming webhook

        hook_id: Incoming webhook GUID

        `Read in Mattermost API docs (webhooks - DeleteIncomingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/DeleteIncomingWebhook>`_

        """
        return self.client.delete(f"/api/v4/hooks/incoming/{hook_id}")

    def update_incoming_webhook(
        self,
        hook_id: str,
        id: str,
        channel_id: str,
        display_name: str,
        description: str,
        username: str | None = None,
        icon_url: str | None = None,
        channel_locked: bool | None = None,
    ):
        """Update an incoming webhook

        hook_id: Incoming Webhook GUID
        id: Incoming webhook GUID
        channel_id: The ID of a public channel or private group that receives the webhook payloads.
        display_name: The display name for this incoming webhook
        description: The description for this incoming webhook
        username: The username this incoming webhook will post as.
        icon_url: The profile picture this incoming webhook will use when posting.
        channel_locked: Whether the webhook is locked to the channel.

        `Read in Mattermost API docs (webhooks - UpdateIncomingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/UpdateIncomingWebhook>`_

        """
        __options = {
            "id": id,
            "channel_id": channel_id,
            "display_name": display_name,
            "description": description,
            "username": username,
            "icon_url": icon_url,
            "channel_locked": channel_locked,
        }
        return self.client.put(f"/api/v4/hooks/incoming/{hook_id}", options=__options)

    def create_outgoing_webhook(
        self,
        team_id: str,
        display_name: str,
        trigger_words: list[str],
        callback_urls: list[str],
        channel_id: str | None = None,
        creator_id: str | None = None,
        description: str | None = None,
        trigger_when: int | None = None,
        content_type: str | None = "application/x-www-form-urlencoded",
    ):
        """Create an outgoing webhook

        team_id: The ID of the team that the webhook watchs
        channel_id: The ID of a public channel that the webhook watchs
        creator_id: The ID of the owner of the webhook if different than the requester. Required in [local mode](https://docs.mattermost.com/administration/mmctl-cli-tool.html#local-mode).
        description: The description for this outgoing webhook
        display_name: The display name for this outgoing webhook
        trigger_words: List of words for the webhook to trigger on
        trigger_when: When to trigger the webhook, ``0`` when a trigger word is present at all and ``1`` if the message starts with a trigger word
        callback_urls: The URLs to POST the payloads to when the webhook is triggered
        content_type: The format to POST the data in, either ``application/json`` or ``application/x-www-form-urlencoded``

        `Read in Mattermost API docs (webhooks - CreateOutgoingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/CreateOutgoingWebhook>`_

        """
        __options = {
            "team_id": team_id,
            "channel_id": channel_id,
            "creator_id": creator_id,
            "description": description,
            "display_name": display_name,
            "trigger_words": trigger_words,
            "trigger_when": trigger_when,
            "callback_urls": callback_urls,
            "content_type": content_type,
        }
        return self.client.post("""/api/v4/hooks/outgoing""", options=__options)

    def get_outgoing_webhooks(
        self, page: int | None = 0, per_page: int | None = 60, team_id: str | None = None, channel_id: str | None = None
    ):
        """List outgoing webhooks

        page: The page to select.
        per_page: The number of hooks per page.
        team_id: The ID of the team to get hooks for.
        channel_id: The ID of the channel to get hooks for.

        `Read in Mattermost API docs (webhooks - GetOutgoingWebhooks) <https://developers.mattermost.com/api-documentation/#/operations/GetOutgoingWebhooks>`_

        """
        __params = {"page": page, "per_page": per_page, "team_id": team_id, "channel_id": channel_id}
        return self.client.get("""/api/v4/hooks/outgoing""", params=__params)

    def get_outgoing_webhook(self, hook_id: str):
        """Get an outgoing webhook

        hook_id: Outgoing webhook GUID

        `Read in Mattermost API docs (webhooks - GetOutgoingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/GetOutgoingWebhook>`_

        """
        return self.client.get(f"/api/v4/hooks/outgoing/{hook_id}")

    def delete_outgoing_webhook(self, hook_id: str):
        """Delete an outgoing webhook

        hook_id: Outgoing webhook GUID

        `Read in Mattermost API docs (webhooks - DeleteOutgoingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/DeleteOutgoingWebhook>`_

        """
        return self.client.delete(f"/api/v4/hooks/outgoing/{hook_id}")

    def update_outgoing_webhook(self, hook_id: str, id: str, channel_id: str, display_name: str, description: str):
        """Update an outgoing webhook

        hook_id: outgoing Webhook GUID
        id: Outgoing webhook GUID
        channel_id: The ID of a public channel or private group that receives the webhook payloads.
        display_name: The display name for this incoming webhook
        description: The description for this incoming webhook

        `Read in Mattermost API docs (webhooks - UpdateOutgoingWebhook) <https://developers.mattermost.com/api-documentation/#/operations/UpdateOutgoingWebhook>`_

        """
        __options = {"id": id, "channel_id": channel_id, "display_name": display_name, "description": description}
        return self.client.put(f"/api/v4/hooks/outgoing/{hook_id}", options=__options)

    def regen_outgoing_hook_token(self, hook_id: str):
        """Regenerate the token for the outgoing webhook.

        hook_id: Outgoing webhook GUID

        `Read in Mattermost API docs (webhooks - RegenOutgoingHookToken) <https://developers.mattermost.com/api-documentation/#/operations/RegenOutgoingHookToken>`_

        """
        return self.client.post(f"/api/v4/hooks/outgoing/{hook_id}/regen_token")



from ._base import Base

__all__ = ["Logs"]


class Logs(Base):

    def download_system_logs(self):
        """Download system logs
        `Read in Mattermost API docs (logs - DownloadSystemLogs) <https://developers.mattermost.com/api-documentation/#/operations/DownloadSystemLogs>`_

        """
        return self.client.get("""/api/v4/logs/download""")



from ._base import Base

__all__ = ["Preferences"]


class Preferences(Base):

    def get_preferences(self, user_id):
        """Get the user's preferences

        user_id: User GUID

        `Read in Mattermost API docs (preferences - GetPreferences) <https://developers.mattermost.com/api-documentation/#/operations/GetPreferences>`_

        """
        return self.client.get(f"/api/v4/users/{user_id}/preferences")

    def update_preferences(self, user_id, options):
        """Save the user's preferences

        user_id: User GUID

        `Read in Mattermost API docs (preferences - UpdatePreferences) <https://developers.mattermost.com/api-documentation/#/operations/UpdatePreferences>`_

        """
        return self.client.put(f"/api/v4/users/{user_id}/preferences", options=options)

    def delete_preferences(self, user_id, options):
        """Delete user's preferences

        user_id: User GUID

        `Read in Mattermost API docs (preferences - DeletePreferences) <https://developers.mattermost.com/api-documentation/#/operations/DeletePreferences>`_

        """
        return self.client.post(f"/api/v4/users/{user_id}/preferences/delete", options=options)

    def get_preferences_by_category(self, user_id, category):
        """List a user's preferences by category

        user_id: User GUID
        category: The category of a group of preferences

        `Read in Mattermost API docs (preferences - GetPreferencesByCategory) <https://developers.mattermost.com/api-documentation/#/operations/GetPreferencesByCategory>`_

        """
        return self.client.get(f"/api/v4/users/{user_id}/preferences/{category}")

    def get_preferences_by_category_by_name(self, user_id, category, preference_name):
        """Get a specific user preference

        user_id: User GUID
        category: The category of a group of preferences
        preference_name: The name of the preference

        `Read in Mattermost API docs (preferences - GetPreferencesByCategoryByName) <https://developers.mattermost.com/api-documentation/#/operations/GetPreferencesByCategoryByName>`_

        """
        return self.client.get(f"/api/v4/users/{user_id}/preferences/{category}/name/{preference_name}")



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Bots"]


class Bots(Base):

    def convert_user_to_bot(self, user_id: str):
        """Convert a user into a bot

        user_id: User GUID

        `Read in Mattermost API docs (bots - ConvertUserToBot) <https://developers.mattermost.com/api-documentation/#/operations/ConvertUserToBot>`_

        """
        return self.client.post(f"/api/v4/users/{user_id}/convert_to_bot")

    def create_bot(self, username: str, display_name: str | None = None, description: str | None = None):
        """Create a bot

        username:
        display_name:
        description:

        `Read in Mattermost API docs (bots - CreateBot) <https://developers.mattermost.com/api-documentation/#/operations/CreateBot>`_

        """
        __options = {"username": username, "display_name": display_name, "description": description}
        return self.client.post("""/api/v4/bots""", options=__options)

    def get_bots(
        self,
        page: int | None = 0,
        per_page: int | None = 60,
        include_deleted: bool | None = None,
        only_orphaned: bool | None = None,
    ):
        """Get bots

        page: The page to select.
        per_page: The number of users per page.
        include_deleted: If deleted bots should be returned.
        only_orphaned: When true, only orphaned bots will be returned. A bot is considered orphaned if its owner has been deactivated.

        `Read in Mattermost API docs (bots - GetBots) <https://developers.mattermost.com/api-documentation/#/operations/GetBots>`_

        """
        __params = {
            "page": page,
            "per_page": per_page,
            "include_deleted": include_deleted,
            "only_orphaned": only_orphaned,
        }
        return self.client.get("""/api/v4/bots""", params=__params)

    def patch_bot(
        self, bot_user_id: str, username: str, display_name: str | None = None, description: str | None = None
    ):
        """Patch a bot

        bot_user_id: Bot user ID
        username:
        display_name:
        description:

        `Read in Mattermost API docs (bots - PatchBot) <https://developers.mattermost.com/api-documentation/#/operations/PatchBot>`_

        """
        __options = {"username": username, "display_name": display_name, "description": description}
        return self.client.put(f"/api/v4/bots/{bot_user_id}", options=__options)

    def get_bot(self, bot_user_id: str, include_deleted: bool | None = None):
        """Get a bot

        bot_user_id: Bot user ID
        include_deleted: If deleted bots should be returned.

        `Read in Mattermost API docs (bots - GetBot) <https://developers.mattermost.com/api-documentation/#/operations/GetBot>`_

        """
        __params = {"include_deleted": include_deleted}
        return self.client.get(f"/api/v4/bots/{bot_user_id}", params=__params)

    def disable_bot(self, bot_user_id: str):
        """Disable a bot

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - DisableBot) <https://developers.mattermost.com/api-documentation/#/operations/DisableBot>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/disable")

    def enable_bot(self, bot_user_id: str):
        """Enable a bot

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - EnableBot) <https://developers.mattermost.com/api-documentation/#/operations/EnableBot>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/enable")

    def assign_bot(self, bot_user_id: str, user_id: str):
        """Assign a bot to a user

        bot_user_id: Bot user ID
        user_id: The user ID to assign the bot to.

        `Read in Mattermost API docs (bots - AssignBot) <https://developers.mattermost.com/api-documentation/#/operations/AssignBot>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/assign/{user_id}")

    def get_bot_icon_image(self, bot_user_id: str):
        """Get bot's LHS icon

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - GetBotIconImage) <https://developers.mattermost.com/api-documentation/#/operations/GetBotIconImage>`_

        """
        return self.client.get(f"/api/v4/bots/{bot_user_id}/icon")

    def set_bot_icon_image(self, bot_user_id: str, image: BinaryIO):
        """Set bot's LHS icon image

        bot_user_id: Bot user ID
        image: SVG icon image to be uploaded

        `Read in Mattermost API docs (bots - SetBotIconImage) <https://developers.mattermost.com/api-documentation/#/operations/SetBotIconImage>`_

        """
        __files = {"image": image}
        return self.client.post(f"/api/v4/bots/{bot_user_id}/icon", files=__files)

    def delete_bot_icon_image(self, bot_user_id: str):
        """Delete bot's LHS icon image

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - DeleteBotIconImage) <https://developers.mattermost.com/api-documentation/#/operations/DeleteBotIconImage>`_

        """
        return self.client.delete(f"/api/v4/bots/{bot_user_id}/icon")

    def convert_bot_to_user(
        self,
        bot_user_id: str,
        email: str | None = None,
        username: str | None = None,
        password: str | None = None,
        first_name: str | None = None,
        last_name: str | None = None,
        nickname: str | None = None,
        locale: str | None = None,
        position: str | None = None,
        props: dict[str, Any] | None = None,
        notify_props: Any | None = None,
    ):
        """Convert a bot into a user

        bot_user_id: Bot user ID
        email:
        username:
        password:
        first_name:
        last_name:
        nickname:
        locale:
        position:
        props:
        notify_props:

        `Read in Mattermost API docs (bots - ConvertBotToUser) <https://developers.mattermost.com/api-documentation/#/operations/ConvertBotToUser>`_

        """
        __options = {
            "email": email,
            "username": username,
            "password": password,
            "first_name": first_name,
            "last_name": last_name,
            "nickname": nickname,
            "locale": locale,
            "position": position,
            "props": props,
            "notify_props": notify_props,
        }
        return self.client.post(f"/api/v4/bots/{bot_user_id}/convert_to_user", options=__options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Authentication"]


class Authentication(Base):

    def migrate_auth_to_ldap(self, from_: str, match_field: str, force: bool):
        """Migrate user accounts authentication type to LDAP.

        from: The current authentication type for the matched users.
        match_field: Foreign user field name to match.
        force:

        `Read in Mattermost API docs (authentication - MigrateAuthToLdap) <https://developers.mattermost.com/api-documentation/#/operations/MigrateAuthToLdap>`_

        """
        __options = {"from": from_, "match_field": match_field, "force": force}
        return self.client.post("""/api/v4/users/migrate_auth/ldap""", options=__options)

    def migrate_auth_to_saml(self, from_: str, matches: dict[str, Any], auto: bool):
        """Migrate user accounts authentication type to SAML.

        from: The current authentication type for the matched users.
        matches: Users map.
        auto:

        `Read in Mattermost API docs (authentication - MigrateAuthToSaml) <https://developers.mattermost.com/api-documentation/#/operations/MigrateAuthToSaml>`_

        """
        __options = {"from": from_, "matches": matches, "auto": auto}
        return self.client.post("""/api/v4/users/migrate_auth/saml""", options=__options)



from ._base import Base

__all__ = ["Cluster"]


class Cluster(Base):

    def get_cluster_status(self):
        """Get cluster status
        `Read in Mattermost API docs (cluster - GetClusterStatus) <https://developers.mattermost.com/api-documentation/#/operations/GetClusterStatus>`_

        """
        return self.client.get("""/api/v4/cluster/status""")



from ._base import Base

__all__ = ["TermsOfService"]


class TermsOfService(Base):

    def register_terms_of_service_action(self, user_id, options):
        """Records user action when they accept or decline custom terms of service

        user_id: User GUID
        serviceTermsId: terms of service ID on which the user is acting on
        accepted: true or false, indicates whether the user accepted or rejected the terms of service.

        `Read in Mattermost API docs (terms_of_service - RegisterTermsOfServiceAction) <https://developers.mattermost.com/api-documentation/#/operations/RegisterTermsOfServiceAction>`_

        """
        return self.client.post(f"/api/v4/users/{user_id}/terms_of_service", options=options)

    def get_user_terms_of_service(self, user_id):
        """Fetches user's latest terms of service action if the latest action was for acceptance.

        user_id: User GUID

        `Read in Mattermost API docs (terms_of_service - GetUserTermsOfService) <https://developers.mattermost.com/api-documentation/#/operations/GetUserTermsOfService>`_

        """
        return self.client.get(f"/api/v4/users/{user_id}/terms_of_service")

    def get_terms_of_service(self):
        """Get latest terms of service
        `Read in Mattermost API docs (terms_of_service - GetTermsOfService) <https://developers.mattermost.com/api-documentation/#/operations/GetTermsOfService>`_

        """
        return self.client.get("""/api/v4/terms_of_service""")

    def create_terms_of_service(self):
        """Creates a new terms of service
        `Read in Mattermost API docs (terms_of_service - CreateTermsOfService) <https://developers.mattermost.com/api-documentation/#/operations/CreateTermsOfService>`_

        """
        return self.client.post("""/api/v4/terms_of_service""")



import asyncio
import logging
import warnings

from ..client import AsyncClient, Client
from ..websocket import Websocket

from .endpoint_base import BaseDriverWithEndpoints, TypedBaseDriverWithEndpoints


log = logging.getLogger("mattermostautodriver.api")
log.setLevel(logging.INFO)


class Driver(BaseDriverWithEndpoints):
    def __new__(cls, options=None, client_cls=Client, *args, **kwargs):
        message = (
            "The old dictionary based API interface is deprecated and will be removed in the future. "
            "Please adapt your code by using the TypedDriver class and expand the arguments you would typically pass. "
            "See https://embl-bio-it.github.io/python-mattermost-autodriver/api_deprecation.html "
            "for additional details."
        )
        warnings.warn(message, DeprecationWarning, stacklevel=2)

        return super().__new__(cls)

    def __init__(self, options=None, client_cls=Client, *args, **kwargs):
        super().__init__(options, client_cls, *args, **kwargs)

    def __enter__(self):
        self.client.__enter__()
        return self

    def __exit__(self, *exc_info):
        return self.client.__exit__(*exc_info)

    def init_websocket(self, event_handler, websocket_cls=Websocket):
        """
        Will initialize the websocket connection to the mattermost server.

        This should be run after login(), because the websocket needs to make
        an authentification.

        See https://developers.mattermost.com/api-documentation/#/websocket for which
        websocket events mattermost sends.

        Example of a really simple event_handler function

        .. code:: python

                async def my_event_handler(message):
                        print(message)


        :param event_handler: The function to handle the websocket events. Takes one argument.
        :type event_handler: Function(message)
        :return: The event loop
        """
        self.websocket = websocket_cls(self.options, self.client.token)
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.websocket.connect(event_handler))
        return loop

    def login(self):
        """
        Logs the user in.

        The log in information is saved in the client
                - userid
                - username
                - cookies

        :return: The raw response from the request
        """
        if self.options["token"]:
            self.client.token = self.options["token"]
            result = self.users.get_user("me")
        else:
            # Note self.user.login() can't be used here as it returns JSON
            # due to self.client.post() pre-defined behavior
            response = self.client.make_request(
                "post",
                "/api/v4/users/login",
                {
                    "login_id": self.options["login_id"],
                    "password": self.options["password"],
                    "token": self.options["mfa_token"],
                },
            )
            if response.status_code == 200:
                self.client.token = response.headers["Token"]
                self.client.cookies = response.cookies
            try:
                result = response.json()
            except ValueError:
                log.debug("Could not convert response to json, returning raw response")
                result = response

        log.debug(result)

        if "id" in result:
            self.client.userid = result["id"]
        if "username" in result:
            self.client.username = result["username"]

        return result

    def logout(self):
        """
        Log the user out.

        :return: The JSON response from the server
        """
        result = self.users.logout()
        self.client.token = ""
        self.client.userid = ""
        self.client.username = ""
        self.client.cookies = None
        return result

    def close(self):
        self.client.close()


class TypedDriver(TypedBaseDriverWithEndpoints):
    def __init__(self, options=None, client_cls=Client, *args, **kwargs):
        super().__init__(options, client_cls, *args, **kwargs)

    def __enter__(self):
        self.client.__enter__()
        return self

    def __exit__(self, *exc_info):
        return self.client.__exit__(*exc_info)

    def init_websocket(self, event_handler, websocket_cls=Websocket):
        """
        Will initialize the websocket connection to the mattermost server.

        This should be run after login(), because the websocket needs to make
        an authentification.

        See https://developers.mattermost.com/api-documentation/#/websocket for which
        websocket events mattermost sends.

        Example of a really simple event_handler function

        .. code:: python

                async def my_event_handler(message):
                        print(message)


        :param event_handler: The function to handle the websocket events. Takes one argument.
        :type event_handler: Function(message)
        :return: The event loop
        """
        self.websocket = websocket_cls(self.options, self.client.token)
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.websocket.connect(event_handler))
        return loop

    def login(self):
        """
        Logs the user in.

        The log in information is saved in the client
                - userid
                - username
                - cookies

        :return: The raw response from the request
        """
        if self.options["token"]:
            self.client.token = self.options["token"]
            result = self.users.get_user("me")
        else:
            # Note self.user.login() can't be used here as it returns JSON
            # due to self.client.post() pre-defined behavior
            response = self.client.make_request(
                "post",
                "/api/v4/users/login",
                {
                    "login_id": self.options["login_id"],
                    "password": self.options["password"],
                    "token": self.options["mfa_token"],
                },
            )
            if response.status_code == 200:
                self.client.token = response.headers["Token"]
                self.client.cookies = response.cookies
            try:
                result = response.json()
            except ValueError:
                log.debug("Could not convert response to json, returning raw response")
                result = response

        log.debug(result)

        if "id" in result:
            self.client.userid = result["id"]
        if "username" in result:
            self.client.username = result["username"]

        return result

    def logout(self):
        """
        Log the user out.

        :return: The JSON response from the server
        """
        result = self.users.logout()
        self.client.token = ""
        self.client.userid = ""
        self.client.username = ""
        self.client.cookies = None
        return result

    def close(self):
        self.client.close()


class AsyncDriver(BaseDriverWithEndpoints):
    def __new__(cls, options=None, client_cls=Client, *args, **kwargs):
        message = (
            "The old dictionary based API interface is deprecated and will be removed in the future. "
            "Please adapt your code by using the AsyncTypedDriver class and expand the arguments you would typically pass. "
            "See https://embl-bio-it.github.io/python-mattermost-autodriver/api_deprecation.html "
            "for additional details."
        )
        warnings.warn(message, DeprecationWarning, stacklevel=2)

        return super().__new__(cls)

    def __init__(self, options=None, client_cls=AsyncClient, *args, **kwargs):
        super().__init__(options, client_cls, *args, **kwargs)

    async def __aenter__(self):
        await self.client.__aenter__()
        return self

    async def __aexit__(self, *exc_info):
        return await self.client.__aexit__(*exc_info)

    def init_websocket(self, event_handler, websocket_cls=Websocket):
        """
        Will initialize the websocket connection to the mattermost server.
        unlike the Driver.init_websocket, this one assumes you are async aware
        and returns a coroutine that can be awaited.  It will not return
        until shutdown() is called.

        This should be run after login(), because the websocket needs to make
        an authentification.

        See https://developers.mattermost.com/api-documentation/#/websocket for which
        websocket events mattermost sends.

        Example of a really simple event_handler function

        .. code:: python

                async def my_event_handler(message):
                        print(message)


        :param event_handler: The function to handle the websocket events. Takes one argument.
        :type event_handler: Function(message)
        :return: coroutine
        """
        self.websocket = websocket_cls(self.options, self.client.token)
        return self.websocket.connect(event_handler)

    async def login(self):
        """
        Logs the user in.

        The log in information is saved in the client
                - userid
                - username
                - cookies

        :return: The raw response from the request
        """
        if self.options["token"]:
            self.client.token = self.options["token"]
            result = await self.users.get_user("me")
        else:
            # Note self.user.login() can't be used here as it returns JSON
            # due to self.client.post() pre-defined behavior
            response = await self.client.make_request(
                "post",
                "/api/v4/users/login",
                {
                    "login_id": self.options["login_id"],
                    "password": self.options["password"],
                    "token": self.options["mfa_token"],
                },
            )
            if response.status_code == 200:
                self.client.token = response.headers["Token"]
                self.client.cookies = response.cookies
            try:
                result = response.json()
            except ValueError:
                log.debug("Could not convert response to json, returning raw response")
                result = response

        log.debug(result)

        if "id" in result:
            self.client.userid = result["id"]
        if "username" in result:
            self.client.username = result["username"]

        return result

    async def logout(self):
        """
        Log the user out.

        :return: The JSON response from the server
        """
        result = await self.users.logout()
        self.client.token = ""
        self.client.userid = ""
        self.client.username = ""
        self.client.cookies = None
        return result

    async def close(self):
        await self.client.close()


class AsyncTypedDriver(TypedBaseDriverWithEndpoints):
    def __init__(self, options=None, client_cls=AsyncClient, *args, **kwargs):
        super().__init__(options, client_cls, *args, **kwargs)

    async def __aenter__(self):
        await self.client.__aenter__()
        return self

    async def __aexit__(self, *exc_info):
        return await self.client.__aexit__(*exc_info)

    def init_websocket(self, event_handler, websocket_cls=Websocket):
        """
        Will initialize the websocket connection to the mattermost server.
        unlike the Driver.init_websocket, this one assumes you are async aware
        and returns a coroutine that can be awaited.  It will not return
        until shutdown() is called.

        This should be run after login(), because the websocket needs to make
        an authentification.

        See https://developers.mattermost.com/api-documentation/#/websocket for which
        websocket events mattermost sends.

        Example of a really simple event_handler function

        .. code:: python

                async def my_event_handler(message):
                        print(message)


        :param event_handler: The function to handle the websocket events. Takes one argument.
        :type event_handler: Function(message)
        :return: coroutine
        """
        self.websocket = websocket_cls(self.options, self.client.token)
        return self.websocket.connect(event_handler)

    async def login(self):
        """
        Logs the user in.

        The log in information is saved in the client
                - userid
                - username
                - cookies

        :return: The raw response from the request
        """
        if self.options["token"]:
            self.client.token = self.options["token"]
            result = await self.users.get_user("me")
        else:
            # Note self.user.login() can't be used here as it returns JSON
            # due to self.client.post() pre-defined behavior
            response = await self.client.make_request(
                "post",
                "/api/v4/users/login",
                {
                    "login_id": self.options["login_id"],
                    "password": self.options["password"],
                    "token": self.options["mfa_token"],
                },
            )
            if response.status_code == 200:
                self.client.token = response.headers["Token"]
                self.client.cookies = response.cookies
            try:
                result = response.json()
            except ValueError:
                log.debug("Could not convert response to json, returning raw response")
                result = response

        log.debug(result)

        if "id" in result:
            self.client.userid = result["id"]
        if "username" in result:
            self.client.username = result["username"]

        return result

    async def logout(self):
        """
        Log the user out.

        :return: The JSON response from the server
        """
        result = await self.users.logout()
        self.client.token = ""
        self.client.userid = ""
        self.client.username = ""
        self.client.cookies = None
        return result

    async def close(self):
        await self.client.close()



from ._base import Base

__all__ = ["OutgoingOauthConnections"]


class OutgoingOauthConnections(Base):

    def list_outgoing_o_auth_connections(self, params=None):
        """List all connections

        team_id: Current Team ID in integrations backstage

        `Read in Mattermost API docs (outgoing_oauth_connections - ListOutgoingOAuthConnections) <https://developers.mattermost.com/api-documentation/#/operations/ListOutgoingOAuthConnections>`_

        """
        return self.client.get("""/api/v4/oauth/outgoing_connections""", params=params)

    def create_outgoing_o_auth_connection(self, options=None):
        """Create a connection
        `Read in Mattermost API docs (outgoing_oauth_connections - CreateOutgoingOAuthConnection) <https://developers.mattermost.com/api-documentation/#/operations/CreateOutgoingOAuthConnection>`_

        """
        return self.client.post("""/api/v4/oauth/outgoing_connections""", options=options)

    def get_outgoing_o_auth_connection(self, params=None):
        """Get a connection

        team_id: Current Team ID in integrations backstage

        `Read in Mattermost API docs (outgoing_oauth_connections - GetOutgoingOAuthConnection) <https://developers.mattermost.com/api-documentation/#/operations/GetOutgoingOAuthConnection>`_

        """
        return self.client.get(f"/api/v4/oauth/outgoing_connections/{connection_id}", params=params)

    def update_outgoing_o_auth_connection(self, options=None):
        """Update a connection
        `Read in Mattermost API docs (outgoing_oauth_connections - UpdateOutgoingOAuthConnection) <https://developers.mattermost.com/api-documentation/#/operations/UpdateOutgoingOAuthConnection>`_

        """
        return self.client.put(f"/api/v4/oauth/outgoing_connections/{connection_id}", options=options)

    def delete_outgoing_o_auth_connection(self):
        """Delete a connection
        `Read in Mattermost API docs (outgoing_oauth_connections - DeleteOutgoingOAuthConnection) <https://developers.mattermost.com/api-documentation/#/operations/DeleteOutgoingOAuthConnection>`_

        """
        return self.client.delete(f"/api/v4/oauth/outgoing_connections/{connection_id}")

    def validate_outgoing_o_auth_connection(self, options=None):
        """Validate a connection configuration
        `Read in Mattermost API docs (outgoing_oauth_connections - ValidateOutgoingOAuthConnection) <https://developers.mattermost.com/api-documentation/#/operations/ValidateOutgoingOAuthConnection>`_

        """
        return self.client.post("""/api/v4/oauth/outgoing_connections/validate""", options=options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["AccessControl"]


class AccessControl(Base):

    def create_access_control_policy(self, options: Any):
        """Create an access control policy
        `Read in Mattermost API docs (access_control - CreateAccessControlPolicy) <https://developers.mattermost.com/api-documentation/#/operations/CreateAccessControlPolicy>`_

        """
        return self.client.put("""/api/v4/access_control_policies""", options=options)

    def check_access_control_policy_expression(self, expression: str | None = None):
        """Check an access control policy expression

        expression: The expression to check.

        `Read in Mattermost API docs (access_control - CheckAccessControlPolicyExpression) <https://developers.mattermost.com/api-documentation/#/operations/CheckAccessControlPolicyExpression>`_

        """
        __options = {"expression": expression}
        return self.client.post("""/api/v4/access_control_policies/cel/check""", options=__options)

    def validate_expression_against_requester(self, expression: str, channelId: str | None = None):
        """Validate if the current user matches a CEL expression

        expression: The CEL expression to validate against the current user.
        channelId: The channel ID for channel-specific permission checks (required for channel admins).

        `Read in Mattermost API docs (access_control - ValidateExpressionAgainstRequester) <https://developers.mattermost.com/api-documentation/#/operations/ValidateExpressionAgainstRequester>`_

        """
        __options = {"expression": expression, "channelId": channelId}
        return self.client.post("""/api/v4/access_control_policies/cel/validate_requester""", options=__options)

    def test_access_control_policy_expression(self, options: Any):
        """Test an access control policy expression
        `Read in Mattermost API docs (access_control - TestAccessControlPolicyExpression) <https://developers.mattermost.com/api-documentation/#/operations/TestAccessControlPolicyExpression>`_

        """
        return self.client.post("""/api/v4/access_control_policies/cel/test""", options=options)

    def search_access_control_policies(self, options: Any):
        """Search access control policies
        `Read in Mattermost API docs (access_control - SearchAccessControlPolicies) <https://developers.mattermost.com/api-documentation/#/operations/SearchAccessControlPolicies>`_

        """
        return self.client.post("""/api/v4/access_control_policies/search""", options=options)

    def get_access_control_policy_autocomplete_fields(self, after: str | None = None, limit: int = 60):
        """Get autocomplete fields for access control policies

        after: The field ID to start after for pagination.
        limit: The maximum number of fields to return.

        `Read in Mattermost API docs (access_control - GetAccessControlPolicyAutocompleteFields) <https://developers.mattermost.com/api-documentation/#/operations/GetAccessControlPolicyAutocompleteFields>`_

        """
        __params = {"after": after, "limit": limit}
        return self.client.get("""/api/v4/access_control_policies/cel/autocomplete/fields""", params=__params)

    def get_access_control_policy(self, policy_id: str):
        """Get an access control policy

        policy_id: The ID of the access control policy.

        `Read in Mattermost API docs (access_control - GetAccessControlPolicy) <https://developers.mattermost.com/api-documentation/#/operations/GetAccessControlPolicy>`_

        """
        return self.client.get(f"/api/v4/access_control_policies/{policy_id}")

    def delete_access_control_policy(self, policy_id: str):
        """Delete an access control policy

        policy_id: The ID of the access control policy.

        `Read in Mattermost API docs (access_control - DeleteAccessControlPolicy) <https://developers.mattermost.com/api-documentation/#/operations/DeleteAccessControlPolicy>`_

        """
        return self.client.delete(f"/api/v4/access_control_policies/{policy_id}")

    def update_access_control_policy_active_status(self, policy_id: str, active: bool):
        """Activate or deactivate an access control policy

        policy_id: The ID of the access control policy.
        active: Set to "true" to activate, "false" to deactivate.

        `Read in Mattermost API docs (access_control - UpdateAccessControlPolicyActiveStatus) <https://developers.mattermost.com/api-documentation/#/operations/UpdateAccessControlPolicyActiveStatus>`_

        """
        __params = {"active": active}
        return self.client.get(f"/api/v4/access_control_policies/{policy_id}/activate", params=__params)

    def assign_access_control_policy_to_channels(self, policy_id: str, channel_ids: list[str] | None = None):
        """Assign an access control policy to channels

        policy_id: The ID of the access control policy.
        channel_ids: The IDs of the channels to assign the policy to.

        `Read in Mattermost API docs (access_control - AssignAccessControlPolicyToChannels) <https://developers.mattermost.com/api-documentation/#/operations/AssignAccessControlPolicyToChannels>`_

        """
        __options = {"channel_ids": channel_ids}
        return self.client.post(f"/api/v4/access_control_policies/{policy_id}/assign", options=__options)

    def unassign_access_control_policy_from_channels(self, policy_id: str, channel_ids: list[str] | None = None):
        """Unassign an access control policy from channels

        policy_id: The ID of the access control policy.
        channel_ids: The IDs of the channels to unassign the policy from.

        `Read in Mattermost API docs (access_control - UnassignAccessControlPolicyFromChannels) <https://developers.mattermost.com/api-documentation/#/operations/UnassignAccessControlPolicyFromChannels>`_

        """
        __params = {"channel_ids": channel_ids}
        return self.client.delete(f"/api/v4/access_control_policies/{policy_id}/unassign", params=__params)

    def get_channels_for_access_control_policy(self, policy_id: str, after: str | None = None, limit: int = 60):
        """Get channels for an access control policy

        policy_id: The ID of the access control policy.
        after: The channel ID to start after for pagination.
        limit: The maximum number of channels to return.

        `Read in Mattermost API docs (access_control - GetChannelsForAccessControlPolicy) <https://developers.mattermost.com/api-documentation/#/operations/GetChannelsForAccessControlPolicy>`_

        """
        __params = {"after": after, "limit": limit}
        return self.client.get(f"/api/v4/access_control_policies/{policy_id}/resources/channels", params=__params)

    def search_channels_for_access_control_policy(self, policy_id: str, options: Any):
        """Search channels for an access control policy

        policy_id: The ID of the access control policy.

        `Read in Mattermost API docs (access_control - SearchChannelsForAccessControlPolicy) <https://developers.mattermost.com/api-documentation/#/operations/SearchChannelsForAccessControlPolicy>`_

        """
        return self.client.post(
            f"/api/v4/access_control_policies/{policy_id}/resources/channels/search", options=options
        )

    def get_channel_access_control_attributes(self, channel_id: str):
        """Get access control attributes for a channel

        channel_id: The ID of the channel.

        `Read in Mattermost API docs (access_control - GetChannelAccessControlAttributes) <https://developers.mattermost.com/api-documentation/#/operations/GetChannelAccessControlAttributes>`_

        """
        return self.client.get(f"/api/v4/channels/{channel_id}/access_control/attributes")

    def get_cel_visual_ast(self, options: Any):
        """Get the visual AST for a CEL expression
        `Read in Mattermost API docs (access_control - GetCELVisualAST) <https://developers.mattermost.com/api-documentation/#/operations/GetCELVisualAST>`_

        """
        return self.client.post("""/api/v4/access_control_policies/cel/visual_ast""", options=options)

    def update_access_control_policies_active(self, options: Any):
        """Activate or deactivate access control policies
        `Read in Mattermost API docs (access_control - UpdateAccessControlPoliciesActive) <https://developers.mattermost.com/api-documentation/#/operations/UpdateAccessControlPoliciesActive>`_

        """
        return self.client.put("""/api/v4/access_control_policies/activate""", options=options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["AuditLogs"]


class AuditLogs(Base):

    def add_audit_log_certificate(self, certificate: BinaryIO):
        """Upload audit log certificate

        certificate: The certificate file

        `Read in Mattermost API docs (audit_logs - AddAuditLogCertificate) <https://developers.mattermost.com/api-documentation/#/operations/AddAuditLogCertificate>`_

        """
        __files = {"certificate": certificate}
        return self.client.post("""/api/v4/audit_logs/certificate""", files=__files)

    def remove_audit_log_certificate(self):
        """Remove audit log certificate
        `Read in Mattermost API docs (audit_logs - RemoveAuditLogCertificate) <https://developers.mattermost.com/api-documentation/#/operations/RemoveAuditLogCertificate>`_

        """
        return self.client.delete("""/api/v4/audit_logs/certificate""")



from ._base import Base

__all__ = ["GroupMessage"]


class GroupMessage(Base):

    def get_group_message_members_common_teams(self, channel_id):
        """Get common teams for members of a Group Message.

        channel_id: Channel GUID

        `Read in Mattermost API docs (group_message - GetGroupMessageMembersCommonTeams) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupMessageMembersCommonTeams>`_

        """
        return self.client.get(f"/api/v4/channels/{channel_id}/common_teams")



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Root"]


class Root(Base):

    def acknowledge_notification(self):
        """Acknowledge receiving of a notification
        `Read in Mattermost API docs (root - AcknowledgeNotification) <https://developers.mattermost.com/api-documentation/#/operations/AcknowledgeNotification>`_

        """
        return self.client.post("""/api/v4/notifications/ack""")



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["IntegrationActions"]


class IntegrationActions(Base):

    def open_interactive_dialog(self, trigger_id: str, url: str, dialog: dict[str, Any]):
        """Open a dialog

        trigger_id: Trigger ID provided by other action
        url: The URL to send the submitted dialog payload to
        dialog: Post object to create

        `Read in Mattermost API docs (integration_actions - OpenInteractiveDialog) <https://developers.mattermost.com/api-documentation/#/operations/OpenInteractiveDialog>`_

        """
        __options = {"trigger_id": trigger_id, "url": url, "dialog": dialog}
        return self.client.post("""/api/v4/actions/dialogs/open""", options=__options)

    def submit_interactive_dialog(
        self,
        url: str,
        channel_id: str,
        team_id: str,
        submission: dict[str, Any],
        callback_id: str | None = None,
        state: str | None = None,
        cancelled: bool | None = None,
    ):
        """Submit a dialog

        url: The URL to send the submitted dialog payload to
        channel_id: Channel ID the user submitted the dialog from
        team_id: Team ID the user submitted the dialog from
        submission: String map where keys are element names and values are the element input values
        callback_id: Callback ID sent when the dialog was opened
        state: State sent when the dialog was opened
        cancelled: Set to true if the dialog was cancelled

        `Read in Mattermost API docs (integration_actions - SubmitInteractiveDialog) <https://developers.mattermost.com/api-documentation/#/operations/SubmitInteractiveDialog>`_

        """
        __options = {
            "url": url,
            "channel_id": channel_id,
            "team_id": team_id,
            "submission": submission,
            "callback_id": callback_id,
            "state": state,
            "cancelled": cancelled,
        }
        return self.client.post("""/api/v4/actions/dialogs/submit""", options=__options)

    def lookup_interactive_dialog(
        self,
        url: str,
        channel_id: str,
        team_id: str,
        submission: dict[str, Any],
        callback_id: str | None = None,
        state: str | None = None,
    ):
        """Lookup dialog elements

        url: The URL to send the lookup request to
        channel_id: Channel ID the user is performing the lookup from
        team_id: Team ID the user is performing the lookup from
        submission: String map where keys are element names and values are the element input values
        callback_id: Callback ID sent when the dialog was opened
        state: State sent when the dialog was opened

        `Read in Mattermost API docs (integration_actions - LookupInteractiveDialog) <https://developers.mattermost.com/api-documentation/#/operations/LookupInteractiveDialog>`_

        """
        __options = {
            "url": url,
            "channel_id": channel_id,
            "team_id": team_id,
            "submission": submission,
            "callback_id": callback_id,
            "state": state,
        }
        return self.client.post("""/api/v4/actions/dialogs/lookup""", options=__options)



from ._base import Base

__all__ = ["Bots"]


class Bots(Base):

    def convert_user_to_bot(self, user_id):
        """Convert a user into a bot

        user_id: User GUID

        `Read in Mattermost API docs (bots - ConvertUserToBot) <https://developers.mattermost.com/api-documentation/#/operations/ConvertUserToBot>`_

        """
        return self.client.post(f"/api/v4/users/{user_id}/convert_to_bot")

    def create_bot(self, options):
        """Create a bot

        username:
        display_name:
        description:

        `Read in Mattermost API docs (bots - CreateBot) <https://developers.mattermost.com/api-documentation/#/operations/CreateBot>`_

        """
        return self.client.post("""/api/v4/bots""", options=options)

    def get_bots(self, params=None):
        """Get bots

        page: The page to select.
        per_page: The number of users per page.
        include_deleted: If deleted bots should be returned.
        only_orphaned: When true, only orphaned bots will be returned. A bot is considered orphaned if its owner has been deactivated.

        `Read in Mattermost API docs (bots - GetBots) <https://developers.mattermost.com/api-documentation/#/operations/GetBots>`_

        """
        return self.client.get("""/api/v4/bots""", params=params)

    def patch_bot(self, bot_user_id, options):
        """Patch a bot

        bot_user_id: Bot user ID
        username:
        display_name:
        description:

        `Read in Mattermost API docs (bots - PatchBot) <https://developers.mattermost.com/api-documentation/#/operations/PatchBot>`_

        """
        return self.client.put(f"/api/v4/bots/{bot_user_id}", options=options)

    def get_bot(self, bot_user_id, params=None):
        """Get a bot

        bot_user_id: Bot user ID
        include_deleted: If deleted bots should be returned.

        `Read in Mattermost API docs (bots - GetBot) <https://developers.mattermost.com/api-documentation/#/operations/GetBot>`_

        """
        return self.client.get(f"/api/v4/bots/{bot_user_id}", params=params)

    def disable_bot(self, bot_user_id):
        """Disable a bot

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - DisableBot) <https://developers.mattermost.com/api-documentation/#/operations/DisableBot>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/disable")

    def enable_bot(self, bot_user_id):
        """Enable a bot

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - EnableBot) <https://developers.mattermost.com/api-documentation/#/operations/EnableBot>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/enable")

    def assign_bot(self, bot_user_id, user_id):
        """Assign a bot to a user

        bot_user_id: Bot user ID
        user_id: The user ID to assign the bot to.

        `Read in Mattermost API docs (bots - AssignBot) <https://developers.mattermost.com/api-documentation/#/operations/AssignBot>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/assign/{user_id}")

    def get_bot_icon_image(self, bot_user_id):
        """Get bot's LHS icon

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - GetBotIconImage) <https://developers.mattermost.com/api-documentation/#/operations/GetBotIconImage>`_

        """
        return self.client.get(f"/api/v4/bots/{bot_user_id}/icon")

    def set_bot_icon_image(self, bot_user_id, files, data=None):
        """Set bot's LHS icon image

        bot_user_id: Bot user ID
        image: SVG icon image to be uploaded

        `Read in Mattermost API docs (bots - SetBotIconImage) <https://developers.mattermost.com/api-documentation/#/operations/SetBotIconImage>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/icon", files=files, data=data)

    def delete_bot_icon_image(self, bot_user_id):
        """Delete bot's LHS icon image

        bot_user_id: Bot user ID

        `Read in Mattermost API docs (bots - DeleteBotIconImage) <https://developers.mattermost.com/api-documentation/#/operations/DeleteBotIconImage>`_

        """
        return self.client.delete(f"/api/v4/bots/{bot_user_id}/icon")

    def convert_bot_to_user(self, bot_user_id, options):
        """Convert a bot into a user

        bot_user_id: Bot user ID
        email:
        username:
        password:
        first_name:
        last_name:
        nickname:
        locale:
        position:
        props:
        notify_props:

        `Read in Mattermost API docs (bots - ConvertBotToUser) <https://developers.mattermost.com/api-documentation/#/operations/ConvertBotToUser>`_

        """
        return self.client.post(f"/api/v4/bots/{bot_user_id}/convert_to_user", options=options)



from ._base import Base

__all__ = ["Recaps"]


class Recaps(Base):

    def create_recap(self, options):
        """Create a channel recap

        title: Title for the recap
        channel_ids: List of channel IDs to include in the recap
        agent_id: ID of the AI agent to use for generating the recap

        `Read in Mattermost API docs (recaps - CreateRecap) <https://developers.mattermost.com/api-documentation/#/operations/CreateRecap>`_

        """
        return self.client.post("""/api/v4/recaps""", options=options)

    def get_recaps_for_user(self, params=None):
        """Get current user's recaps

        page: The page to select.
        per_page: The number of recaps per page.

        `Read in Mattermost API docs (recaps - GetRecapsForUser) <https://developers.mattermost.com/api-documentation/#/operations/GetRecapsForUser>`_

        """
        return self.client.get("""/api/v4/recaps""", params=params)

    def get_recap(self, recap_id):
        """Get a specific recap

        recap_id: Recap GUID

        `Read in Mattermost API docs (recaps - GetRecap) <https://developers.mattermost.com/api-documentation/#/operations/GetRecap>`_

        """
        return self.client.get(f"/api/v4/recaps/{recap_id}")

    def delete_recap(self, recap_id):
        """Delete a recap

        recap_id: Recap GUID

        `Read in Mattermost API docs (recaps - DeleteRecap) <https://developers.mattermost.com/api-documentation/#/operations/DeleteRecap>`_

        """
        return self.client.delete(f"/api/v4/recaps/{recap_id}")

    def mark_recap_as_read(self, recap_id):
        """Mark a recap as read

        recap_id: Recap GUID

        `Read in Mattermost API docs (recaps - MarkRecapAsRead) <https://developers.mattermost.com/api-documentation/#/operations/MarkRecapAsRead>`_

        """
        return self.client.post(f"/api/v4/recaps/{recap_id}/read")

    def regenerate_recap(self, recap_id):
        """Regenerate a recap

        recap_id: Recap GUID

        `Read in Mattermost API docs (recaps - RegenerateRecap) <https://developers.mattermost.com/api-documentation/#/operations/RegenerateRecap>`_

        """
        return self.client.post(f"/api/v4/recaps/{recap_id}/regenerate")



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Exports"]


class Exports(Base):

    def list_exports(self):
        """List export files
        `Read in Mattermost API docs (exports - ListExports) <https://developers.mattermost.com/api-documentation/#/operations/ListExports>`_

        """
        return self.client.get("""/api/v4/exports""")

    def download_export(self, export_name: str):
        """Download an export file

        export_name: The name of the export file to download

        `Read in Mattermost API docs (exports - DownloadExport) <https://developers.mattermost.com/api-documentation/#/operations/DownloadExport>`_

        """
        return self.client.get(f"/api/v4/exports/{export_name}")

    def delete_export(self, export_name: str):
        """Delete an export file

        export_name: The name of the export file to delete

        `Read in Mattermost API docs (exports - DeleteExport) <https://developers.mattermost.com/api-documentation/#/operations/DeleteExport>`_

        """
        return self.client.delete(f"/api/v4/exports/{export_name}")



from ._base import Base

__all__ = ["Posts"]


class Posts(Base):

    def create_post(self, options):
        """Create a post

        channel_id: The channel ID to post in
        message: The message contents, can be formatted with Markdown
        root_id: The post ID to comment on
        file_ids: A list of file IDs to associate with the post. Note that posts are limited to 5 files maximum. Please use additional posts for more files.
        props: A general JSON property bag to attach to the post
        metadata: A JSON object to add post metadata, e.g the post's priority

        `Read in Mattermost API docs (posts - CreatePost) <https://developers.mattermost.com/api-documentation/#/operations/CreatePost>`_

        """
        return self.client.post("""/api/v4/posts""", options=options)

    def create_post_ephemeral(self, options):
        """Create a ephemeral post

        user_id: The target user id for the ephemeral post
        post: Post object to create

        `Read in Mattermost API docs (posts - CreatePostEphemeral) <https://developers.mattermost.com/api-documentation/#/operations/CreatePostEphemeral>`_

        """
        return self.client.post("""/api/v4/posts/ephemeral""", options=options)

    def get_post(self, post_id, params=None):
        """Get a post

        post_id: ID of the post to get
        include_deleted: Defines if result should include deleted posts, must have 'manage_system' (admin) permission.

        `Read in Mattermost API docs (posts - GetPost) <https://developers.mattermost.com/api-documentation/#/operations/GetPost>`_

        """
        return self.client.get(f"/api/v4/posts/{post_id}", params=params)

    def delete_post(self, post_id):
        """Delete a post

        post_id: ID of the post to delete

        `Read in Mattermost API docs (posts - DeletePost) <https://developers.mattermost.com/api-documentation/#/operations/DeletePost>`_

        """
        return self.client.delete(f"/api/v4/posts/{post_id}")

    def update_post(self, post_id, options):
        """Update a post

        post_id: ID of the post to update
        id: ID of the post to update
        is_pinned: Set to ``true`` to pin the post to the channel it is in
        message: The message text of the post
        has_reactions: Set to ``true`` if the post has reactions to it
        props: A general JSON property bag to attach to the post

        `Read in Mattermost API docs (posts - UpdatePost) <https://developers.mattermost.com/api-documentation/#/operations/UpdatePost>`_

        """
        return self.client.put(f"/api/v4/posts/{post_id}", options=options)

    def set_post_unread(self, user_id, post_id):
        """Mark as unread from a post.

        user_id: User GUID
        post_id: Post GUID

        `Read in Mattermost API docs (posts - SetPostUnread) <https://developers.mattermost.com/api-documentation/#/operations/SetPostUnread>`_

        """
        return self.client.post(f"/api/v4/users/{user_id}/posts/{post_id}/set_unread")

    def patch_post(self, post_id, options):
        """Patch a post

        post_id: Post GUID
        is_pinned: Set to ``true`` to pin the post to the channel it is in
        message: The message text of the post
        file_ids: The list of files attached to this post
        has_reactions: Set to ``true`` if the post has reactions to it
        props: A general JSON property bag to attach to the post

        `Read in Mattermost API docs (posts - PatchPost) <https://developers.mattermost.com/api-documentation/#/operations/PatchPost>`_

        """
        return self.client.put(f"/api/v4/posts/{post_id}/patch", options=options)

    def get_post_thread(self, post_id, params=None):
        """Get a thread

        post_id: ID of a post in the thread
        perPage: The number of posts per page
        fromPost: The post_id to return the next page of posts from
        fromCreateAt: The create_at timestamp to return the next page of posts from
        fromUpdateAt: The update_at timestamp to return the next page of posts from. You cannot set this flag with direction=down.
        direction: The direction to return the posts. Either up or down.
        skipFetchThreads: Whether to skip fetching threads or not
        collapsedThreads: Whether the client uses CRT or not
        collapsedThreadsExtended: Whether to return the associated users as part of the response or not
        updatesOnly: This flag is used to make the API work with the updateAt value. If you set this flag, you must set a value for fromUpdateAt.

        `Read in Mattermost API docs (posts - GetPostThread) <https://developers.mattermost.com/api-documentation/#/operations/GetPostThread>`_

        """
        return self.client.get(f"/api/v4/posts/{post_id}/thread", params=params)

    def get_flagged_posts_for_user(self, user_id, params=None):
        """Get a list of flagged posts

        user_id: ID of the user
        team_id: Team ID
        channel_id: Channel ID
        page: The page to select
        per_page: The number of posts per page

        `Read in Mattermost API docs (posts - GetFlaggedPostsForUser) <https://developers.mattermost.com/api-documentation/#/operations/GetFlaggedPostsForUser>`_

        """
        return self.client.get(f"/api/v4/users/{user_id}/posts/flagged", params=params)

    def get_file_infos_for_post(self, post_id, params=None):
        """Get file info for post

        post_id: ID of the post
        include_deleted: Defines if result should include deleted posts, must have 'manage_system' (admin) permission.

        `Read in Mattermost API docs (posts - GetFileInfosForPost) <https://developers.mattermost.com/api-documentation/#/operations/GetFileInfosForPost>`_

        """
        return self.client.get(f"/api/v4/posts/{post_id}/files/info", params=params)

    def get_posts_for_channel(self, channel_id, params=None):
        """Get posts for a channel

        channel_id: The channel ID to get the posts for
        page: The page to select
        per_page: The number of posts per page
        since: Provide a non-zero value in Unix time milliseconds to select posts modified after that time
        before: A post id to select the posts that came before this one
        after: A post id to select the posts that came after this one
        include_deleted: Whether to include deleted posts or not. Must have system admin permissions.

        `Read in Mattermost API docs (posts - GetPostsForChannel) <https://developers.mattermost.com/api-documentation/#/operations/GetPostsForChannel>`_

        """
        return self.client.get(f"/api/v4/channels/{channel_id}/posts", params=params)

    def get_posts_around_last_unread(self, user_id, channel_id, params=None):
        """Get posts around oldest unread

        user_id: ID of the user
        channel_id: The channel ID to get the posts for
        limit_before: Number of posts before the oldest unread posts. Maximum is 200 posts if limit is set greater than that.
        limit_after: Number of posts after and including the oldest unread post. Maximum is 200 posts if limit is set greater than that.
        skipFetchThreads: Whether to skip fetching threads or not
        collapsedThreads: Whether the client uses CRT or not
        collapsedThreadsExtended: Whether to return the associated users as part of the response or not

        `Read in Mattermost API docs (posts - GetPostsAroundLastUnread) <https://developers.mattermost.com/api-documentation/#/operations/GetPostsAroundLastUnread>`_

        """
        return self.client.get(f"/api/v4/users/{user_id}/channels/{channel_id}/posts/unread", params=params)

    def search_posts(self, team_id, options):
        """Search for team posts

        team_id: Team GUID
        terms: The search terms as inputed by the user. To search for posts from a user include ``from:someusername``, using a user's username. To search in a specific channel include ``in:somechannel``, using the channel name (not the display name).
        is_or_search: Set to true if an Or search should be performed vs an And search.
        time_zone_offset: Offset from UTC of user timezone for date searches.
        include_deleted_channels: Set to true if deleted channels should be included in the search. (archived channels)
        page: The page to select. (Only works with Elasticsearch)
        per_page: The number of posts per page. (Only works with Elasticsearch)

        `Read in Mattermost API docs (posts - SearchPosts) <https://developers.mattermost.com/api-documentation/#/operations/SearchPosts>`_

        """
        return self.client.post(f"/api/v4/teams/{team_id}/posts/search", options=options)

    def pin_post(self, post_id):
        """Pin a post to the channel

        post_id: Post GUID

        `Read in Mattermost API docs (posts - PinPost) <https://developers.mattermost.com/api-documentation/#/operations/PinPost>`_

        """
        return self.client.post(f"/api/v4/posts/{post_id}/pin")

    def unpin_post(self, post_id):
        """Unpin a post to the channel

        post_id: Post GUID

        `Read in Mattermost API docs (posts - UnpinPost) <https://developers.mattermost.com/api-documentation/#/operations/UnpinPost>`_

        """
        return self.client.post(f"/api/v4/posts/{post_id}/unpin")

    def do_post_action(self, post_id, action_id):
        """Perform a post action

        post_id: Post GUID
        action_id: Action GUID

        `Read in Mattermost API docs (posts - DoPostAction) <https://developers.mattermost.com/api-documentation/#/operations/DoPostAction>`_

        """
        return self.client.post(f"/api/v4/posts/{post_id}/actions/{action_id}")

    def get_posts_by_ids(self, options):
        """Get posts by a list of ids
        `Read in Mattermost API docs (posts - getPostsByIds) <https://developers.mattermost.com/api-documentation/#/operations/getPostsByIds>`_

        """
        return self.client.post("""/api/v4/posts/ids""", options=options)

    def set_post_reminder(self, user_id, post_id, options):
        """Set a post reminder

        user_id: User GUID
        post_id: Post GUID
        target_time: Target time for the reminder

        `Read in Mattermost API docs (posts - SetPostReminder) <https://developers.mattermost.com/api-documentation/#/operations/SetPostReminder>`_

        """
        return self.client.post(f"/api/v4/users/{user_id}/posts/{post_id}/reminder", options=options)

    def save_acknowledgement_for_post(self, user_id, post_id):
        """Acknowledge a post

        user_id: User GUID
        post_id: Post GUID

        `Read in Mattermost API docs (posts - SaveAcknowledgementForPost) <https://developers.mattermost.com/api-documentation/#/operations/SaveAcknowledgementForPost>`_

        """
        return self.client.post(f"/api/v4/users/{user_id}/posts/{post_id}/ack")

    def delete_acknowledgement_for_post(self, user_id, post_id):
        """Delete a post acknowledgement

        user_id: User GUID
        post_id: Post GUID

        `Read in Mattermost API docs (posts - DeleteAcknowledgementForPost) <https://developers.mattermost.com/api-documentation/#/operations/DeleteAcknowledgementForPost>`_

        """
        return self.client.delete(f"/api/v4/users/{user_id}/posts/{post_id}/ack")

    def move_thread(self, post_id, options):
        """Move a post (and any posts within that post's thread)

        post_id: The identifier of the post to move
        channel_id: The channel identifier of where the post/thread is to be moved

        `Read in Mattermost API docs (posts - MoveThread) <https://developers.mattermost.com/api-documentation/#/operations/MoveThread>`_

        """
        return self.client.post(f"/api/v4/posts/{post_id}/move", options=options)

    def restore_post_version(self, post_id, restore_version_id):
        """Restores a past version of a post

        post_id: The identifier of the post to restore
        restore_version_id: The identifier of the past version of post to restore to

        `Read in Mattermost API docs (posts - RestorePostVersion) <https://developers.mattermost.com/api-documentation/#/operations/RestorePostVersion>`_

        """
        return self.client.post(f"/api/v4/posts/{post_id}/restore/{restore_version_id}")

    def reveal_post(self, post_id):
        """Reveal a burn-on-read post

        post_id: The identifier of the post to reveal

        `Read in Mattermost API docs (posts - RevealPost) <https://developers.mattermost.com/api-documentation/#/operations/RevealPost>`_

        """
        return self.client.get(f"/api/v4/posts/{post_id}/reveal")

    def burn_post(self, post_id):
        """Burn a burn-on-read post

        post_id: The identifier of the post to burn

        `Read in Mattermost API docs (posts - BurnPost) <https://developers.mattermost.com/api-documentation/#/operations/BurnPost>`_

        """
        return self.client.delete(f"/api/v4/posts/{post_id}/burn")

    def rewrite_message(self, options):
        """Rewrite a message using AI

        agent_id: The ID of the AI agent to use for rewriting
        message: The message text to rewrite
        action: The rewrite action to perform
        custom_prompt: Custom prompt for rewriting. Required when action is "custom", optional otherwise.

        `Read in Mattermost API docs (posts - RewriteMessage) <https://developers.mattermost.com/api-documentation/#/operations/RewriteMessage>`_

        """
        return self.client.post("""/api/v4/posts/rewrite""", options=options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Reports"]


class Reports(Base):

    def get_users_for_reporting(
        self,
        sort_column: str | None = "Username",
        direction: str | None = "next",
        sort_direction: str | None = "asc",
        page_size: int | None = 50,
        from_column_value: str | None = None,
        from_id: str | None = None,
        date_range: str | None = "alltime",
        role_filter: str | None = None,
        team_filter: str | None = None,
        has_no_team: bool | None = None,
        hide_active: bool | None = None,
        hide_inactive: bool | None = None,
        search_term: str | None = None,
    ):
        """Get a list of paged and sorted users for admin reporting purposes

        sort_column: The column to sort the users by. Must be one of ("CreateAt", "Username", "FirstName", "LastName", "Nickname", "Email") or the API will return an error.
        direction: The direction to accept paging values from. Will return values ahead of the cursor if "prev", and below the cursor if "next". Default is "next".
        sort_direction: The sorting direction. Must be one of ("asc", "desc"). Will default to 'asc' if not specified or the input is invalid.
        page_size: The maximum number of users to return.
        from_column_value: The value of the sorted column corresponding to the cursor to read from. Should be blank for the first page asked for.
        from_id: The value of the user id corresponding to the cursor to read from. Should be blank for the first page asked for.
        date_range: The date range of the post statistics to display. Must be one of ("last30days", "previousmonth", "last6months", "alltime"). Will default to 'alltime' if the input is not valid.
        role_filter: Filter users by their role.
        team_filter: Filter users by a specified team ID.
        has_no_team: If true, show only users that have no team. Will ignore provided "team_filter" if true.
        hide_active: If true, show only users that are inactive. Cannot be used at the same time as "hide_inactive"
        hide_inactive: If true, show only users that are active. Cannot be used at the same time as "hide_active"
        search_term: A filtering search term that allows filtering by Username, FirstName, LastName, Nickname or Email

        `Read in Mattermost API docs (reports - GetUsersForReporting) <https://developers.mattermost.com/api-documentation/#/operations/GetUsersForReporting>`_

        """
        __params = {
            "sort_column": sort_column,
            "direction": direction,
            "sort_direction": sort_direction,
            "page_size": page_size,
            "from_column_value": from_column_value,
            "from_id": from_id,
            "date_range": date_range,
            "role_filter": role_filter,
            "team_filter": team_filter,
            "has_no_team": has_no_team,
            "hide_active": hide_active,
            "hide_inactive": hide_inactive,
            "search_term": search_term,
        }
        return self.client.get("""/api/v4/reports/users""", params=__params)

    def get_user_count_for_reporting(
        self,
        role_filter: str | None = None,
        team_filter: str | None = None,
        has_no_team: bool | None = None,
        hide_active: bool | None = None,
        hide_inactive: bool | None = None,
        search_term: str | None = None,
    ):
        """Gets the full count of users that match the filter.

        role_filter: Filter users by their role.
        team_filter: Filter users by a specified team ID.
        has_no_team: If true, show only users that have no team. Will ignore provided "team_filter" if true.
        hide_active: If true, show only users that are inactive. Cannot be used at the same time as "hide_inactive"
        hide_inactive: If true, show only users that are active. Cannot be used at the same time as "hide_active"
        search_term: A filtering search term that allows filtering by Username, FirstName, LastName, Nickname or Email

        `Read in Mattermost API docs (reports - GetUserCountForReporting) <https://developers.mattermost.com/api-documentation/#/operations/GetUserCountForReporting>`_

        """
        __params = {
            "role_filter": role_filter,
            "team_filter": team_filter,
            "has_no_team": has_no_team,
            "hide_active": hide_active,
            "hide_inactive": hide_inactive,
            "search_term": search_term,
        }
        return self.client.get("""/api/v4/reports/users/count""", params=__params)

    def start_batch_users_export(self):
        """Starts a job to export the users to a report file.
        `Read in Mattermost API docs (reports - StartBatchUsersExport) <https://developers.mattermost.com/api-documentation/#/operations/StartBatchUsersExport>`_

        """
        return self.client.post("""/api/v4/reports/users/export""")

    def get_posts_for_reporting(
        self,
        channel_id: str,
        cursor: str | None = "",
        start_time: int | None = None,
        time_field: str | None = "create_at",
        sort_direction: str | None = "asc",
        per_page: int | None = 100,
        include_deleted: bool | None = False,
        exclude_system_posts: bool | None = False,
        include_metadata: bool | None = False,
    ):
        """Get posts for reporting and compliance purposes using cursor-based pagination

        channel_id: The ID of the channel to retrieve posts from
        cursor: Opaque cursor string for pagination. Omit or use empty string for the first request. For subsequent requests, use the exact cursor value from the previous response's next_cursor. The cursor is base64-encoded and contains all pagination state including time, post ID, and query parameters. Do not attempt to parse or modify the cursor value.

        start_time: Optional start time for query range in Unix milliseconds. Only used for the first request (ignored when cursor is provided). - For "asc" (ascending): starts retrieving from this time going forward - For "desc" (descending): starts retrieving from this time going backward If omitted, defaults to 0 for ascending or MaxInt64 for descending.

        time_field: Which timestamp field to use for sorting and filtering. Use "create_at" to retrieve posts by creation time, or "update_at" to retrieve posts by last modification time.

        sort_direction: Sort direction for pagination. Use "asc" to retrieve posts from oldest to newest, or "desc" to retrieve from newest to oldest.

        per_page: Number of posts to return per page. Maximum 1000.
        include_deleted: If true, include posts that have been deleted (DeleteAt > 0). By default, only non-deleted posts are returned.

        exclude_system_posts: If true, exclude all system posts.

        include_metadata: If true, enrich posts with additional metadata including file information, reactions, custom emojis, priority, and acknowledgements. Note that this may increase response time for large result sets.


        `Read in Mattermost API docs (reports - GetPostsForReporting) <https://developers.mattermost.com/api-documentation/#/operations/GetPostsForReporting>`_

        """
        __options = {
            "channel_id": channel_id,
            "cursor": cursor,
            "start_time": start_time,
            "time_field": time_field,
            "sort_direction": sort_direction,
            "per_page": per_page,
            "include_deleted": include_deleted,
            "exclude_system_posts": exclude_system_posts,
            "include_metadata": include_metadata,
        }
        return self.client.post("""/api/v4/reports/posts""", options=__options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Saml"]


class Saml(Base):

    def migrate_auth_to_saml(self, from_: str, matches: dict[str, Any], auto: bool):
        """Migrate user accounts authentication type to SAML.

        from: The current authentication type for the matched users.
        matches: Users map.
        auto:

        `Read in Mattermost API docs (saml - MigrateAuthToSaml) <https://developers.mattermost.com/api-documentation/#/operations/MigrateAuthToSaml>`_

        """
        __options = {"from": from_, "matches": matches, "auto": auto}
        return self.client.post("""/api/v4/users/migrate_auth/saml""", options=__options)

    def get_saml_metadata(self):
        """Get metadata
        `Read in Mattermost API docs (saml - GetSamlMetadata) <https://developers.mattermost.com/api-documentation/#/operations/GetSamlMetadata>`_

        """
        return self.client.get("""/api/v4/saml/metadata""")

    def get_saml_metadata_from_idp(self, saml_metadata_url: str | None = None):
        """Get metadata from Identity Provider

        saml_metadata_url: The URL from which to retrieve the SAML IDP data.

        `Read in Mattermost API docs (saml - GetSamlMetadataFromIdp) <https://developers.mattermost.com/api-documentation/#/operations/GetSamlMetadataFromIdp>`_

        """
        __options = {"saml_metadata_url": saml_metadata_url}
        return self.client.post("""/api/v4/saml/metadatafromidp""", options=__options)

    def upload_saml_idp_certificate(self, certificate: BinaryIO):
        """Upload IDP certificate

        certificate: The IDP certificate file

        `Read in Mattermost API docs (saml - UploadSamlIdpCertificate) <https://developers.mattermost.com/api-documentation/#/operations/UploadSamlIdpCertificate>`_

        """
        __files = {"certificate": certificate}
        return self.client.post("""/api/v4/saml/certificate/idp""", files=__files)

    def delete_saml_idp_certificate(self):
        """Remove IDP certificate
        `Read in Mattermost API docs (saml - DeleteSamlIdpCertificate) <https://developers.mattermost.com/api-documentation/#/operations/DeleteSamlIdpCertificate>`_

        """
        return self.client.delete("""/api/v4/saml/certificate/idp""")

    def upload_saml_public_certificate(self, certificate: BinaryIO):
        """Upload public certificate

        certificate: The public certificate file

        `Read in Mattermost API docs (saml - UploadSamlPublicCertificate) <https://developers.mattermost.com/api-documentation/#/operations/UploadSamlPublicCertificate>`_

        """
        __files = {"certificate": certificate}
        return self.client.post("""/api/v4/saml/certificate/public""", files=__files)

    def delete_saml_public_certificate(self):
        """Remove public certificate
        `Read in Mattermost API docs (saml - DeleteSamlPublicCertificate) <https://developers.mattermost.com/api-documentation/#/operations/DeleteSamlPublicCertificate>`_

        """
        return self.client.delete("""/api/v4/saml/certificate/public""")

    def upload_saml_private_certificate(self, certificate: BinaryIO):
        """Upload private key

        certificate: The private key file

        `Read in Mattermost API docs (saml - UploadSamlPrivateCertificate) <https://developers.mattermost.com/api-documentation/#/operations/UploadSamlPrivateCertificate>`_

        """
        __files = {"certificate": certificate}
        return self.client.post("""/api/v4/saml/certificate/private""", files=__files)

    def delete_saml_private_certificate(self):
        """Remove private key
        `Read in Mattermost API docs (saml - DeleteSamlPrivateCertificate) <https://developers.mattermost.com/api-documentation/#/operations/DeleteSamlPrivateCertificate>`_

        """
        return self.client.delete("""/api/v4/saml/certificate/private""")

    def get_saml_certificate_status(self):
        """Get certificate status
        `Read in Mattermost API docs (saml - GetSamlCertificateStatus) <https://developers.mattermost.com/api-documentation/#/operations/GetSamlCertificateStatus>`_

        """
        return self.client.get("""/api/v4/saml/certificate/status""")

    def reset_saml_auth_data_to_email(
        self, include_deleted: bool | None = False, dry_run: bool | None = False, user_ids: list[str] | None = []
    ):
        """Reset AuthData to Email

        include_deleted: Whether to include deleted users.
        dry_run: If set to true, the number of users who would be affected is returned.
        user_ids: If set to a non-empty array, then users whose IDs are not in the array will be excluded.

        `Read in Mattermost API docs (saml - ResetSamlAuthDataToEmail) <https://developers.mattermost.com/api-documentation/#/operations/ResetSamlAuthDataToEmail>`_

        """
        __options = {"include_deleted": include_deleted, "dry_run": dry_run, "user_ids": user_ids}
        return self.client.post("""/api/v4/saml/reset_auth_data""", options=__options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Groups"]


class Groups(Base):

    def unlink_ldap_group(self, remote_id: str):
        """Delete a link for LDAP group

        remote_id: Group GUID

        `Read in Mattermost API docs (groups - UnlinkLdapGroup) <https://developers.mattermost.com/api-documentation/#/operations/UnlinkLdapGroup>`_

        """
        return self.client.delete(f"/api/v4/ldap/groups/{remote_id}/link")

    def get_groups(
        self,
        page: int | None = 0,
        per_page: int | None = 60,
        q: str | None = None,
        include_member_count: bool | None = None,
        not_associated_to_team: str | None = None,
        not_associated_to_channel: str | None = None,
        since: int | None = None,
        filter_allow_reference: bool | None = False,
    ):
        """Get groups

        page: The page to select.
        per_page: The number of groups per page.
        q: String to pattern match the ``name`` and ``display_name`` field. Will return all groups whose ``name`` and ``display_name`` field match any of the text.
        include_member_count: Boolean which adds the ``member_count`` attribute to each group JSON object
        not_associated_to_team: Team GUID which is used to return all the groups not associated to this team
        not_associated_to_channel: Group GUID which is used to return all the groups not associated to this channel
        since: Only return groups that have been modified since the given Unix timestamp (in milliseconds). All modified groups, including deleted and created groups, will be returned.
        *Minimum server version*: 5.24

        filter_allow_reference: Boolean which filters the group entries with the ``allow_reference`` attribute set.

        `Read in Mattermost API docs (groups - GetGroups) <https://developers.mattermost.com/api-documentation/#/operations/GetGroups>`_

        """
        __params = {
            "page": page,
            "per_page": per_page,
            "q": q,
            "include_member_count": include_member_count,
            "not_associated_to_team": not_associated_to_team,
            "not_associated_to_channel": not_associated_to_channel,
            "since": since,
            "filter_allow_reference": filter_allow_reference,
        }
        return self.client.get("""/api/v4/groups""", params=__params)

    def create_group(self, name: str, display_name: str, source: str, allow_reference: bool, user_ids: list[str]):
        """Create a custom group

        name: The unique group name used for at-mentioning.
        display_name: The display name of the group which can include spaces.
        source: Must be ``custom``
        allow_reference: Must be true
        user_ids: The user ids of the group members to add.

        `Read in Mattermost API docs (groups - CreateGroup) <https://developers.mattermost.com/api-documentation/#/operations/CreateGroup>`_

        """
        __options = {
            "name": name,
            "display_name": display_name,
            "source": source,
            "allow_reference": allow_reference,
            "user_ids": user_ids,
        }
        return self.client.post("""/api/v4/groups""", options=__options)

    def get_group(self, group_id: str):
        """Get a group

        group_id: Group GUID

        `Read in Mattermost API docs (groups - GetGroup) <https://developers.mattermost.com/api-documentation/#/operations/GetGroup>`_

        """
        return self.client.get(f"/api/v4/groups/{group_id}")

    def delete_group(self, group_id: str):
        """Deletes a custom group

        group_id: The ID of the group.

        `Read in Mattermost API docs (groups - DeleteGroup) <https://developers.mattermost.com/api-documentation/#/operations/DeleteGroup>`_

        """
        return self.client.delete(f"/api/v4/groups/{group_id}")

    def patch_group(
        self, group_id: str, name: str | None = None, display_name: str | None = None, description: str | None = None
    ):
        """Patch a group

        group_id: Group GUID
        name:
        display_name:
        description:

        `Read in Mattermost API docs (groups - PatchGroup) <https://developers.mattermost.com/api-documentation/#/operations/PatchGroup>`_

        """
        __options = {"name": name, "display_name": display_name, "description": description}
        return self.client.put(f"/api/v4/groups/{group_id}/patch", options=__options)

    def restore_group(self, group_id: str):
        """Restore a previously deleted group.

        group_id: Group GUID

        `Read in Mattermost API docs (groups - RestoreGroup) <https://developers.mattermost.com/api-documentation/#/operations/RestoreGroup>`_

        """
        return self.client.post(f"/api/v4/groups/{group_id}/restore")

    def link_group_syncable_for_team(self, group_id: str, team_id: str):
        """Link a team to a group

        group_id: Group GUID
        team_id: Team GUID

        `Read in Mattermost API docs (groups - LinkGroupSyncableForTeam) <https://developers.mattermost.com/api-documentation/#/operations/LinkGroupSyncableForTeam>`_

        """
        return self.client.post(f"/api/v4/groups/{group_id}/teams/{team_id}/link")

    def unlink_group_syncable_for_team(self, group_id: str, team_id: str):
        """Delete a link from a team to a group

        group_id: Group GUID
        team_id: Team GUID

        `Read in Mattermost API docs (groups - UnlinkGroupSyncableForTeam) <https://developers.mattermost.com/api-documentation/#/operations/UnlinkGroupSyncableForTeam>`_

        """
        return self.client.delete(f"/api/v4/groups/{group_id}/teams/{team_id}/link")

    def link_group_syncable_for_channel(self, group_id: str, channel_id: str):
        """Link a channel to a group

        group_id: Group GUID
        channel_id: Channel GUID

        `Read in Mattermost API docs (groups - LinkGroupSyncableForChannel) <https://developers.mattermost.com/api-documentation/#/operations/LinkGroupSyncableForChannel>`_

        """
        return self.client.post(f"/api/v4/groups/{group_id}/channels/{channel_id}/link")

    def unlink_group_syncable_for_channel(self, group_id: str, channel_id: str):
        """Delete a link from a channel to a group

        group_id: Group GUID
        channel_id: Channel GUID

        `Read in Mattermost API docs (groups - UnlinkGroupSyncableForChannel) <https://developers.mattermost.com/api-documentation/#/operations/UnlinkGroupSyncableForChannel>`_

        """
        return self.client.delete(f"/api/v4/groups/{group_id}/channels/{channel_id}/link")

    def get_group_syncable_for_team_id(self, group_id: str, team_id: str):
        """Get GroupSyncable from Team ID

        group_id: Group GUID
        team_id: Team GUID

        `Read in Mattermost API docs (groups - GetGroupSyncableForTeamId) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupSyncableForTeamId>`_

        """
        return self.client.get(f"/api/v4/groups/{group_id}/teams/{team_id}")

    def get_group_syncable_for_channel_id(self, group_id: str, channel_id: str):
        """Get GroupSyncable from channel ID

        group_id: Group GUID
        channel_id: Channel GUID

        `Read in Mattermost API docs (groups - GetGroupSyncableForChannelId) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupSyncableForChannelId>`_

        """
        return self.client.get(f"/api/v4/groups/{group_id}/channels/{channel_id}")

    def get_group_syncables_teams(self, group_id: str):
        """Get group teams

        group_id: Group GUID

        `Read in Mattermost API docs (groups - GetGroupSyncablesTeams) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupSyncablesTeams>`_

        """
        return self.client.get(f"/api/v4/groups/{group_id}/teams")

    def get_group_syncables_channels(self, group_id: str):
        """Get group channels

        group_id: Group GUID

        `Read in Mattermost API docs (groups - GetGroupSyncablesChannels) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupSyncablesChannels>`_

        """
        return self.client.get(f"/api/v4/groups/{group_id}/channels")

    def patch_group_syncable_for_team(self, group_id: str, team_id: str, auto_add: bool | None = None):
        """Patch a GroupSyncable associated to Team

        group_id: Group GUID
        team_id: Team GUID
        auto_add:

        `Read in Mattermost API docs (groups - PatchGroupSyncableForTeam) <https://developers.mattermost.com/api-documentation/#/operations/PatchGroupSyncableForTeam>`_

        """
        __options = {"auto_add": auto_add}
        return self.client.put(f"/api/v4/groups/{group_id}/teams/{team_id}/patch", options=__options)

    def patch_group_syncable_for_channel(self, group_id: str, channel_id: str, auto_add: bool | None = None):
        """Patch a GroupSyncable associated to Channel

        group_id: Group GUID
        channel_id: Channel GUID
        auto_add:

        `Read in Mattermost API docs (groups - PatchGroupSyncableForChannel) <https://developers.mattermost.com/api-documentation/#/operations/PatchGroupSyncableForChannel>`_

        """
        __options = {"auto_add": auto_add}
        return self.client.put(f"/api/v4/groups/{group_id}/channels/{channel_id}/patch", options=__options)

    def get_group_users(self, group_id: str, page: int | None = 0, per_page: int | None = 60):
        """Get group users

        group_id: Group GUID
        page: The page to select.
        per_page: The number of groups per page.

        `Read in Mattermost API docs (groups - GetGroupUsers) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupUsers>`_

        """
        __params = {"page": page, "per_page": per_page}
        return self.client.get(f"/api/v4/groups/{group_id}/members", params=__params)

    def delete_group_members(self, group_id: str, user_ids: list[str] | None = None):
        """Removes members from a custom group

        group_id: The ID of the group to delete.
        user_ids:

        `Read in Mattermost API docs (groups - DeleteGroupMembers) <https://developers.mattermost.com/api-documentation/#/operations/DeleteGroupMembers>`_

        """
        __params = {"user_ids": user_ids}
        return self.client.delete(f"/api/v4/groups/{group_id}/members", params=__params)

    def add_group_members(self, group_id: str, user_ids: list[str] | None = None):
        """Adds members to a custom group

        group_id: The ID of the group.
        user_ids:

        `Read in Mattermost API docs (groups - AddGroupMembers) <https://developers.mattermost.com/api-documentation/#/operations/AddGroupMembers>`_

        """
        __options = {"user_ids": user_ids}
        return self.client.post(f"/api/v4/groups/{group_id}/members", options=__options)

    def get_group_stats(self, group_id: str):
        """Get group stats

        group_id: Group GUID

        `Read in Mattermost API docs (groups - GetGroupStats) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupStats>`_

        """
        return self.client.get(f"/api/v4/groups/{group_id}/stats")

    def get_groups_by_channel(
        self,
        channel_id: str,
        page: int | None = 0,
        per_page: int | None = 60,
        filter_allow_reference: bool | None = False,
    ):
        """Get channel groups

        channel_id: Channel GUID
        page: The page to select.
        per_page: The number of groups per page.
        filter_allow_reference: Boolean which filters the group entries with the ``allow_reference`` attribute set.

        `Read in Mattermost API docs (groups - GetGroupsByChannel) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupsByChannel>`_

        """
        __params = {"page": page, "per_page": per_page, "filter_allow_reference": filter_allow_reference}
        return self.client.get(f"/api/v4/channels/{channel_id}/groups", params=__params)

    def get_groups_by_team(
        self,
        team_id: str,
        page: int | None = 0,
        per_page: int | None = 60,
        filter_allow_reference: bool | None = False,
        include_member_count: bool | None = False,
        include_timezones: bool | None = False,
        include_total_count: bool | None = False,
        include_archived: bool | None = False,
        filter_archived: bool | None = False,
        filter_parent_team_permitted: bool | None = False,
        filter_has_member: str | None = None,
        include_member_ids: bool | None = False,
        only_syncable_sources: bool | None = False,
    ):
        """Get team groups

        team_id: Team GUID
        page: The page to select.
        per_page: The number of groups per page.
        filter_allow_reference: Boolean which filters in the group entries with the ``allow_reference`` attribute set.
        include_member_count: Boolean which adds a ``member_count`` field to each group object.
        include_timezones: Boolean which adds timezone information for group members.
        include_total_count: Boolean which adds total count of groups in the response.
        include_archived: Boolean which includes archived groups in the response.
        filter_archived: Boolean which filters out archived groups from the response.
        filter_parent_team_permitted: Boolean which filters groups based on parent team permissions.
        filter_has_member: User ID to filter groups that have this member.
        include_member_ids: Boolean which adds member IDs to the group objects.
        only_syncable_sources: Boolean which includes groups from syncable sources.

        `Read in Mattermost API docs (groups - GetGroupsByTeam) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupsByTeam>`_

        """
        __params = {
            "page": page,
            "per_page": per_page,
            "filter_allow_reference": filter_allow_reference,
            "include_member_count": include_member_count,
            "include_timezones": include_timezones,
            "include_total_count": include_total_count,
            "include_archived": include_archived,
            "filter_archived": filter_archived,
            "filter_parent_team_permitted": filter_parent_team_permitted,
            "filter_has_member": filter_has_member,
            "include_member_ids": include_member_ids,
            "only_syncable_sources": only_syncable_sources,
        }
        return self.client.get(f"/api/v4/teams/{team_id}/groups", params=__params)

    def get_groups_associated_to_channels_by_team(
        self,
        team_id: str,
        page: int | None = 0,
        per_page: int | None = 60,
        filter_allow_reference: bool | None = False,
        paginate: bool | None = False,
    ):
        """Get team groups by channels

        team_id: Team GUID
        page: The page to select.
        per_page: The number of groups per page.
        filter_allow_reference: Boolean which filters in the group entries with the ``allow_reference`` attribute set.
        paginate: Boolean to determine whether the pagination should be applied or not

        `Read in Mattermost API docs (groups - GetGroupsAssociatedToChannelsByTeam) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupsAssociatedToChannelsByTeam>`_

        """
        __params = {
            "page": page,
            "per_page": per_page,
            "filter_allow_reference": filter_allow_reference,
            "paginate": paginate,
        }
        return self.client.get(f"/api/v4/teams/{team_id}/groups_by_channels", params=__params)

    def get_groups_by_user_id(self, user_id: str):
        """Get groups for a userId

        user_id: User GUID

        `Read in Mattermost API docs (groups - GetGroupsByUserId) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupsByUserId>`_

        """
        return self.client.get(f"/api/v4/users/{user_id}/groups")

    def get_groups_by_names(self, options: list[str]):
        """Get groups by name
        `Read in Mattermost API docs (groups - GetGroupsByNames) <https://developers.mattermost.com/api-documentation/#/operations/GetGroupsByNames>`_

        """
        return self.client.post("""/api/v4/groups/names""", options=options)



from ._base import Base
from typing import Any, BinaryIO

__all__ = ["Ai"]


class Ai(Base):

    def create_recap(self, title: str, channel_ids: list[str], agent_id: str):
        """Create a channel recap

        title: Title for the recap
        channel_ids: List of channel IDs to include in the recap
        agent_id: ID of the AI agent to use for generating the recap

        `Read in Mattermost API docs (ai - CreateRecap) <https://developers.mattermost.com/api-documentation/#/operations/CreateRecap>`_

        """
        __options = {"title": title, "channel_ids": channel_ids, "agent_id": agent_id}
        return self.client.post("""/api/v4/recaps""", options=__options)

    def get_recaps_for_user(self, page: int | None = 0, per_page: int | None = 60):
        """Get current user's recaps

        page: The page to select.
        per_page: The number of recaps per page.

        `Read in Mattermost API docs (ai - GetRecapsForUser) <https://developers.mattermost.com/api-documentation/#/operations/GetRecapsForUser>`_

        """
        __params = {"page": page, "per_page": per_page}
        return self.client.get("""/api/v4/recaps""", params=__params)

    def get_recap(self, recap_id: str):
        """Get a specific recap

        recap_id: Recap GUID

        `Read in Mattermost API docs (ai - GetRecap) <https://developers.mattermost.com/api-documentation/#/operations/GetRecap>`_

        """
        return self.client.get(f"/api/v4/recaps/{recap_id}")

    def delete_recap(self, recap_id: str):
        """Delete a recap

        recap_id: Recap GUID

        `Read in Mattermost API docs (ai - DeleteRecap) <https://developers.mattermost.com/api-documentation/#/operations/DeleteRecap>`_

        """
        return self.client.delete(f"/api/v4/recaps/{recap_id}")

    def mark_recap_as_read(self, recap_id: str):
        """Mark a recap as read

        recap_id: Recap GUID

        `Read in Mattermost API docs (ai - MarkRecapAsRead) <https://developers.mattermost.com/api-documentation/#/operations/MarkRecapAsRead>`_

        """
        return self.client.post(f"/api/v4/recaps/{recap_id}/read")

    def regenerate_recap(self, recap_id: str):
        """Regenerate a recap

        recap_id: Recap GUID

        `Read in Mattermost API docs (ai - RegenerateRecap) <https://developers.mattermost.com/api-documentation/#/operations/RegenerateRecap>`_

        """
        return self.client.post(f"/api/v4/recaps/{recap_id}/regenerate")

    def get_ai_agents(self):
        """Get available AI agents
        `Read in Mattermost API docs (ai - GetAIAgents) <https://developers.mattermost.com/api-documentation/#/operations/GetAIAgents>`_

        """
        return self.client.get("""/api/v4/ai/agents""")

    def get_ai_services(self):
        """Get available AI services
        `Read in Mattermost API docs (ai - GetAIServices) <https://developers.mattermost.com/api-documentation/#/operations/GetAIServices>`_

        """
        return self.client.get("""/api/v4/ai/services""")



import json
import ssl
import asyncio
import logging
import time

import aiohttp

log = logging.getLogger("mattermostautodriver.websocket")
log.setLevel(logging.INFO)


class Websocket:
    def __init__(self, options, token):
        self.options = options
        if options["debug"]:
            log.setLevel(logging.DEBUG)
        self._token = token
        self._alive = False
        self._last_msg = 0

    async def connect(self, event_handler):
        """
        Connect to the websocket and authenticate it.
        When the authentication has finished, start the loop listening for messages,
        sending a ping to the server to keep the connection alive.

        :param event_handler: Every websocket event will be passed there. Takes one argument.
        :type event_handler: Function(message)
        :return:
        """
        context = ssl.create_default_context(purpose=ssl.Purpose.SERVER_AUTH)
        if not self.options["verify"]:
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE

        scheme = "wss://"
        if self.options["scheme"] != "https":
            scheme = "ws://"
            context = None

        url = "{scheme:s}{url:s}:{port:s}/api/v4/websocket".format(
            scheme=scheme, url=self.options["url"], port=str(self.options["port"])
        )

        self._alive = True

        while True:
            try:
                kw_args = {}
                if self.options["websocket_kw_args"] is not None:
                    kw_args = self.options["websocket_kw_args"]
                async with aiohttp.ClientSession() as session:
                    async with session.ws_connect(
                        url,
                        ssl=context,
                        proxy=self.options["proxy"],
                        **kw_args,
                    ) as websocket:
                        await self._authenticate_websocket(websocket, event_handler)
                        while self._alive:
                            try:
                                await self._start_loop(websocket, event_handler)
                            except aiohttp.ClientError:
                                break
                        if (not self.options["keepalive"]) or (not self._alive):
                            break
            except Exception as e:
                log.exception(f"Failed to establish websocket connection: {type(e)} thrown")
                await asyncio.sleep(self.options["keepalive_delay"])

    async def _start_loop(self, websocket, event_handler):
        """
        We will listen for websockets events, sending a heartbeats on a timer.
        If we don't the webserver would close the idle connection,
        forcing us to reconnect.
        """
        log.debug("Starting websocket loop")
        # TODO: move to create_task when cpython 3.7 is minimum supported python version
        keep_alive = asyncio.create_task(self._do_heartbeats(websocket))
        log.debug("Waiting for messages on websocket")
        while self._alive:
            message = await websocket.receive_str()
            self._last_msg = time.time()
            await event_handler(message)
        log.debug("cancelling heartbeat task")
        keep_alive.cancel()
        try:
            await keep_alive
        except asyncio.CancelledError:
            pass

    async def _do_heartbeats(self, websocket):
        """
        This is a little complicated, but we only need to pong the websocket if
        we haven't recieved a message inside the timeout window.

        Since messages can be received, while we are waiting we need to check
        after sleep.
        """
        timeout = self.options["timeout"]
        while True:
            since_last_msg = time.time() - self._last_msg
            next_timeout = timeout - since_last_msg if since_last_msg <= timeout else timeout
            await asyncio.sleep(next_timeout)
            if time.time() - self._last_msg >= timeout:
                log.debug("sending heartbeat...")
                await websocket.pong()
                self._last_msg = time.time()

    def disconnect(self):
        """Sets ``self._alive`` to False so the loop in ``self._start_loop`` will finish."""
        log.info("Disconnecting websocket")
        self._alive = False

    async def _authenticate_websocket(self, websocket, event_handler):
        """
        Sends a authentication challenge over a websocket.
        This is not needed when we just send the cookie we got on login
        when connecting to the websocket.
        """
        log.debug("Authenticating websocket")
        json_data = json.dumps({"seq": 1, "action": "authentication_challenge", "data": {"token": self._token}})
        await websocket.send_str(json_data)
        while True:
            message = await websocket.receive_str()
            status = json.loads(message)
            log.debug(status)
            # We want to pass the events to the event_handler already
            # because the hello event could arrive before the authentication ok response
            await event_handler(message)
            if ("event" in status and status["event"] == "hello") and ("seq" in status and status["seq"] == 0):
                log.info("Websocket authentication OK")
                return True
            log.error("Websocket authentication failed")



from .base import BaseDriver
from ..client import Client
from ..endpoints.access_control import AccessControl
from ..endpoints.agents import Agents
from ..endpoints.ai import Ai
from ..endpoints.audit_logs import AuditLogs
from ..endpoints.authentication import Authentication
from ..endpoints.bleve import Bleve
from ..endpoints.bookmarks import Bookmarks
from ..endpoints.bots import Bots
from ..endpoints.brand import Brand
from ..endpoints.channels import Channels
from ..endpoints.cloud import Cloud
from ..endpoints.cluster import Cluster
from ..endpoints.commands import Commands
from ..endpoints.compliance import Compliance
from ..endpoints.conditions import Conditions
from ..endpoints.custom_profile_attributes import CustomProfileAttributes
from ..endpoints.data_retention import DataRetention
from ..endpoints.elasticsearch import Elasticsearch
from ..endpoints.emoji import Emoji
from ..endpoints.exports import Exports
from ..endpoints.files import Files
from ..endpoints.filtering import Filtering
from ..endpoints.group_message import GroupMessage
from ..endpoints.groups import Groups
from ..endpoints.imports import Imports
from ..endpoints.integration_actions import IntegrationActions
from ..endpoints.internal import Internal
from ..endpoints.ip import Ip
from ..endpoints.jobs import Jobs
from ..endpoints.ldap import Ldap
from ..endpoints.logs import Logs
from ..endpoints.metrics import Metrics
from ..endpoints.migrate import Migrate
from ..endpoints.o_auth import OAuth
from ..endpoints.oauth import Oauth
from ..endpoints.outgoing_connections import OutgoingConnections
from ..endpoints.outgoing_oauth_connections import OutgoingOauthConnections
from ..endpoints.permissions import Permissions
from ..endpoints.playbook_autofollows import PlaybookAutofollows
from ..endpoints.playbook_runs import PlaybookRuns
from ..endpoints.playbooks import Playbooks
from ..endpoints.plugins import Plugins
from ..endpoints.posts import Posts
from ..endpoints.preferences import Preferences
from ..endpoints.reactions import Reactions
from ..endpoints.recaps import Recaps
from ..endpoints.remote_clusters import RemoteClusters
from ..endpoints.reports import Reports
from ..endpoints.roles import Roles
from ..endpoints.root import Root
from ..endpoints.saml import Saml
from ..endpoints.scheduled_post import ScheduledPost
from ..endpoints.schemes import Schemes
from ..endpoints.search import Search
from ..endpoints.shared_channels import SharedChannels
from ..endpoints.status import Status
from ..endpoints.system import System
from ..endpoints.teams import Teams
from ..endpoints.terms_of_service import TermsOfService
from ..endpoints.threads import Threads
from ..endpoints.timeline import Timeline
from ..endpoints.uploads import Uploads
from ..endpoints.usage import Usage
from ..endpoints.users import Users
from ..endpoints.webhooks import Webhooks
from ..endpoints_old.access_control import AccessControl as OldAccessControl
from ..endpoints_old.agents import Agents as OldAgents
from ..endpoints_old.ai import Ai as OldAi
from ..endpoints_old.audit_logs import AuditLogs as OldAuditLogs
from ..endpoints_old.authentication import Authentication as OldAuthentication
from ..endpoints_old.bleve import Bleve as OldBleve
from ..endpoints_old.bookmarks import Bookmarks as OldBookmarks
from ..endpoints_old.bots import Bots as OldBots
from ..endpoints_old.brand import Brand as OldBrand
from ..endpoints_old.channels import Channels as OldChannels
from ..endpoints_old.cloud import Cloud as OldCloud
from ..endpoints_old.cluster import Cluster as OldCluster
from ..endpoints_old.commands import Commands as OldCommands
from ..endpoints_old.compliance import Compliance as OldCompliance
from ..endpoints_old.conditions import Conditions as OldConditions
from ..endpoints_old.custom_profile_attributes import CustomProfileAttributes as OldCustomProfileAttributes
from ..endpoints_old.data_retention import DataRetention as OldDataRetention
from ..endpoints_old.elasticsearch import Elasticsearch as OldElasticsearch
from ..endpoints_old.emoji import Emoji as OldEmoji
from ..endpoints_old.exports import Exports as OldExports
from ..endpoints_old.files import Files as OldFiles
from ..endpoints_old.filtering import Filtering as OldFiltering
from ..endpoints_old.group_message import GroupMessage as OldGroupMessage
from ..endpoints_old.groups import Groups as OldGroups
from ..endpoints_old.imports import Imports as OldImports
from ..endpoints_old.integration_actions import IntegrationActions as OldIntegrationActions
from ..endpoints_old.internal import Internal as OldInternal
from ..endpoints_old.ip import Ip as OldIp
from ..endpoints_old.jobs import Jobs as OldJobs
from ..endpoints_old.ldap import Ldap as OldLdap
from ..endpoints_old.logs import Logs as OldLogs
from ..endpoints_old.metrics import Metrics as OldMetrics
from ..endpoints_old.migrate import Migrate as OldMigrate
from ..endpoints_old.o_auth import OAuth as OldOAuth
from ..endpoints_old.oauth import Oauth as OldOauth
from ..endpoints_old.outgoing_connections import OutgoingConnections as OldOutgoingConnections
from ..endpoints_old.outgoing_oauth_connections import OutgoingOauthConnections as OldOutgoingOauthConnections
from ..endpoints_old.permissions import Permissions as OldPermissions
from ..endpoints_old.playbook_autofollows import PlaybookAutofollows as OldPlaybookAutofollows
from ..endpoints_old.playbook_runs import PlaybookRuns as OldPlaybookRuns
from ..endpoints_old.playbooks import Playbooks as OldPlaybooks
from ..endpoints_old.plugins import Plugins as OldPlugins
from ..endpoints_old.posts import Posts as OldPosts
from ..endpoints_old.preferences import Preferences as OldPreferences
from ..endpoints_old.reactions import Reactions as OldReactions
from ..endpoints_old.recaps import Recaps as OldRecaps
from ..endpoints_old.remote_clusters import RemoteClusters as OldRemoteClusters
from ..endpoints_old.reports import Reports as OldReports
from ..endpoints_old.roles import Roles as OldRoles
from ..endpoints_old.root import Root as OldRoot
from ..endpoints_old.saml import Saml as OldSaml
from ..endpoints_old.scheduled_post import ScheduledPost as OldScheduledPost
from ..endpoints_old.schemes import Schemes as OldSchemes
from ..endpoints_old.search import Search as OldSearch
from ..endpoints_old.shared_channels import SharedChannels as OldSharedChannels
from ..endpoints_old.status import Status as OldStatus
from ..endpoints_old.system import System as OldSystem
from ..endpoints_old.teams import Teams as OldTeams
from ..endpoints_old.terms_of_service import TermsOfService as OldTermsOfService
from ..endpoints_old.threads import Threads as OldThreads
from ..endpoints_old.timeline import Timeline as OldTimeline
from ..endpoints_old.uploads import Uploads as OldUploads
from ..endpoints_old.usage import Usage as OldUsage
from ..endpoints_old.users import Users as OldUsers
from ..endpoints_old.webhooks import Webhooks as OldWebhooks


class BaseDriverWithEndpoints(BaseDriver):

    def __init__(self, options=None, client_cls=Client, *args, **kwargs):
        super().__init__(options, client_cls, *args, **kwargs)
        self.access_control = OldAccessControl(self.client)
        self.agents = OldAgents(self.client)
        self.ai = OldAi(self.client)
        self.audit_logs = OldAuditLogs(self.client)
        self.authentication = OldAuthentication(self.client)
        self.bleve = OldBleve(self.client)
        self.bookmarks = OldBookmarks(self.client)
        self.bots = OldBots(self.client)
        self.brand = OldBrand(self.client)
        self.channels = OldChannels(self.client)
        self.cloud = OldCloud(self.client)
        self.cluster = OldCluster(self.client)
        self.commands = OldCommands(self.client)
        self.compliance = OldCompliance(self.client)
        self.conditions = OldConditions(self.client)
        self.custom_profile_attributes = OldCustomProfileAttributes(self.client)
        self.data_retention = OldDataRetention(self.client)
        self.elasticsearch = OldElasticsearch(self.client)
        self.emoji = OldEmoji(self.client)
        self.exports = OldExports(self.client)
        self.files = OldFiles(self.client)
        self.filtering = OldFiltering(self.client)
        self.group_message = OldGroupMessage(self.client)
        self.groups = OldGroups(self.client)
        self.imports = OldImports(self.client)
        self.integration_actions = OldIntegrationActions(self.client)
        self.internal = OldInternal(self.client)
        self.ip = OldIp(self.client)
        self.jobs = OldJobs(self.client)
        self.ldap = OldLdap(self.client)
        self.logs = OldLogs(self.client)
        self.metrics = OldMetrics(self.client)
        self.migrate = OldMigrate(self.client)
        self.o_auth = OldOAuth(self.client)
        self.oauth = OldOauth(self.client)
        self.outgoing_connections = OldOutgoingConnections(self.client)
        self.outgoing_oauth_connections = OldOutgoingOauthConnections(self.client)
        self.permissions = OldPermissions(self.client)
        self.playbook_autofollows = OldPlaybookAutofollows(self.client)
        self.playbook_runs = OldPlaybookRuns(self.client)
        self.playbooks = OldPlaybooks(self.client)
        self.plugins = OldPlugins(self.client)
        self.posts = OldPosts(self.client)
        self.preferences = OldPreferences(self.client)
        self.reactions = OldReactions(self.client)
        self.recaps = OldRecaps(self.client)
        self.remote_clusters = OldRemoteClusters(self.client)
        self.reports = OldReports(self.client)
        self.roles = OldRoles(self.client)
        self.root = OldRoot(self.client)
        self.saml = OldSaml(self.client)
        self.scheduled_post = OldScheduledPost(self.client)
        self.schemes = OldSchemes(self.client)
        self.search = OldSearch(self.client)
        self.shared_channels = OldSharedChannels(self.client)
        self.status = OldStatus(self.client)
        self.system = OldSystem(self.client)
        self.teams = OldTeams(self.client)
        self.terms_of_service = OldTermsOfService(self.client)
        self.threads = OldThreads(self.client)
        self.timeline = OldTimeline(self.client)
        self.uploads = OldUploads(self.client)
        self.usage = OldUsage(self.client)
        self.users = OldUsers(self.client)
        self.webhooks = OldWebhooks(self.client)


class TypedBaseDriverWithEndpoints(BaseDriver):

    def __init__(self, options=None, client_cls=Client, *args, **kwargs):
        super().__init__(options, client_cls, *args, **kwargs)
        self.access_control = AccessControl(self.client)
        self.agents = Agents(self.client)
        self.ai = Ai(self.client)
        self.audit_logs = AuditLogs(self.client)
        self.authentication = Authentication(self.client)
        self.bleve = Bleve(self.client)
        self.bookmarks = Bookmarks(self.client)
        self.bots = Bots(self.client)
        self.brand = Brand(self.client)
        self.channels = Channels(self.client)
        self.cloud = Cloud(self.client)
        self.cluster = Cluster(self.client)
        self.commands = Commands(self.client)
        self.compliance = Compliance(self.client)
        self.conditions = Conditions(self.client)
        self.custom_profile_attributes = CustomProfileAttributes(self.client)
        self.data_retention = DataRetention(self.client)
        self.elasticsearch = Elasticsearch(self.client)
        self.emoji = Emoji(self.client)
        self.exports = Exports(self.client)
        self.files = Files(self.client)
        self.filtering = Filtering(self.client)
        self.group_message = GroupMessage(self.client)
        self.groups = Groups(self.client)
        self.imports = Imports(self.client)
        self.integration_actions = IntegrationActions(self.client)
        self.internal = Internal(self.client)
        self.ip = Ip(self.client)
        self.jobs = Jobs(self.client)
        self.ldap = Ldap(self.client)
        self.logs = Logs(self.client)
        self.metrics = Metrics(self.client)
        self.migrate = Migrate(self.client)
        self.o_auth = OAuth(self.client)
        self.oauth = Oauth(self.client)
        self.outgoing_connections = OutgoingConnections(self.client)
        self.outgoing_oauth_connections = OutgoingOauthConnections(self.client)
        self.permissions = Permissions(self.client)
        self.playbook_autofollows = PlaybookAutofollows(self.client)
        self.playbook_runs = PlaybookRuns(self.client)
        self.playbooks = Playbooks(self.client)
        self.plugins = Plugins(self.client)
        self.posts = Posts(self.client)
        self.preferences = Preferences(self.client)
        self.reactions = Reactions(self.client)
        self.recaps = Recaps(self.client)
        self.remote_clusters = RemoteClusters(self.client)
        self.reports = Reports(self.client)
        self.roles = Roles(self.client)
        self.root = Root(self.client)
        self.saml = Saml(self.client)
        self.scheduled_post = ScheduledPost(self.client)
        self.schemes = Schemes(self.client)
        self.search = Search(self.client)
        self.shared_channels = SharedChannels(self.client)
        self.status = Status(self.client)
        self.system = System(self.client)
        self.teams = Teams(self.client)
        self.terms_of_service = TermsOfService(self.client)
        self.threads = Threads(self.client)
        self.timeline = Timeline(self.client)
        self.uploads = Uploads(self.client)
        self.usage = Usage(self.client)
        self.users = Users(self.client)
        self.webhooks = Webhooks(self.client)



from httpx import HTTPError


class InvalidMattermostError(Exception):
    """
    Raised when mattermost returns an invalid error
    """

    def __init__(self, message: str, status_code: int):
        super().__init__(message)
        self.status_code: int = status_code


class MattermostError(HTTPError):
    """
    Base class for all known mattermost errors
    """

    def __init__(
        self,
        message: str,
        status_code: int,
        error_id: str,
        request_id: str,
        is_oauth_error: bool,
    ):
        super().__init__(message)
        self.status_code: int = status_code
        self.error_id: str = error_id
        self.request_id: str = request_id
        self.is_oauth_error: bool = is_oauth_error


class UnknownMattermostError(MattermostError):
    """
    Raised when mattermost returns a status code that is not known
    """


class InvalidOrMissingParameters(MattermostError):
    """
    Raised when mattermost returns a
    400 Invalid or missing parameters in URL or request body
    """

    def __init__(self, message: str, error_id: str, request_id: str, is_oauth_error: bool):
        super().__init__(
            message=message,
            status_code=400,
            error_id=error_id,
            request_id=request_id,
            is_oauth_error=is_oauth_error,
        )


class NoAccessTokenProvided(MattermostError):
    """
    Raised when mattermost returns a
    401 No access token provided
    """

    def __init__(self, message: str, error_id: str, request_id: str, is_oauth_error: bool):
        super().__init__(
            message=message,
            status_code=401,
            error_id=error_id,
            request_id=request_id,
            is_oauth_error=is_oauth_error,
        )


class NotEnoughPermissions(MattermostError):
    """
    Raised when mattermost returns a
    403 Do not have appropriate permissions
    """

    def __init__(self, message: str, error_id: str, request_id: str, is_oauth_error: bool):
        super().__init__(
            message=message,
            status_code=403,
            error_id=error_id,
            request_id=request_id,
            is_oauth_error=is_oauth_error,
        )


class ResourceNotFound(MattermostError):
    """
    Raised when mattermost returns a
    404 Resource not found
    """

    def __init__(self, message: str, error_id: str, request_id: str, is_oauth_error: bool):
        super().__init__(
            message=message,
            status_code=404,
            error_id=error_id,
            request_id=request_id,
            is_oauth_error=is_oauth_error,
        )


class MethodNotAllowed(MattermostError):
    """
    Raised when mattermost returns a
    405 Method Not Allowed
    """

    def __init__(self, message: str, error_id: str, request_id: str, is_oauth_error: bool):
        super().__init__(
            message=message,
            status_code=405,
            error_id=error_id,
            request_id=request_id,
            is_oauth_error=is_oauth_error,
        )


class ContentTooLarge(MattermostError):
    """
    Raised when mattermost returns a
    413 Content too large
    """

    def __init__(self, message: str, error_id: str, request_id: str, is_oauth_error: bool):
        super().__init__(
            message=message,
            status_code=413,
            error_id=error_id,
            request_id=request_id,
            is_oauth_error=is_oauth_error,
        )


class FeatureDisabled(MattermostError):
    """
    Raised when mattermost returns a
    501 Feature is disabled
    """

    def __init__(self, message: str, error_id: str, request_id: str, is_oauth_error: bool):
        super().__init__(
            message=message,
            status_code=501,
            error_id=error_id,
            request_id=request_id,
            is_oauth_error=is_oauth_error,
        )
